version 14
adopath + ../external/lib/stata/gslab_misc/ado 
adopath + ../external/lib/stata/ado

preliminaries
set more off

global pre_data "../external/pre_pps/pre_pps_data"
global post_data "../external/compiled/CompiledStayLevelData"

program main
    drgweights
    discharges_by_los_dest
    discharges_weighted
    graph_discharges_by_los_dest
    graph_discharges_weighted
end

program drgweights
    foreach period in pre post {
        use LTCDRG_Code using ../temp/${`period'_data}, clear
        gen num_stays_`period'=1
        gen total_stays_`period' = _N
        collapse (sum) num_stays_`period' (mean) total_stays_`period', by(LTCDRG_Code)
        gen drgweight_`period' = num_stays_`period'/total_stays_`period'
        save ../temp/drgweights_`period', replace
    }
    
    use LTCDRG_Code drgweight_pre using ../temp/drgweights_pre, clear
    merge 1:1 LTCDRG_Code using ../temp/drgweights_post, assert(1 2 3) keep(1 2 3) ///
        keepusing(drgweight_post)
    count if _merge==1
    local pre_nomatch = `r(N)'
    count if _merge==3 | _merge==1
    local pre_nomatch_frac = `pre_nomatch'/`r(N)'
    di "fraction of pre period DRGs not matched to post is `pre_nomatch_frac'"
    egen pre_nomatch_wt = total(drgweight_pre) if _merge==1
    sum pre_nomatch_wt
    di "total of pre-period missing weight is `r(mean)'"
    count if _merge==2
    local post_nomatch = `r(N)'
    count if _merge==3 | _merge==2
    local post_nomatch_frac = `post_nomatch'/`r(N)'
    di "fraction of post period DRGs not matched to pre is `post_nomatch_frac'"
    egen post_nomatch_wt = total(drgweight_post) if _merge==2
    sum post_nomatch_wt
    di "total of post-period missing weight is `r(mean)'"
    drop _merge

    gen drgweight = drgweight_post/drgweight_pre
    replace drgweight = 0 if missing(drgweight)
    keep LTCDRG_Code drgweight
    save ../temp/drgweights, replace
end

program discharges_by_los_dest
    foreach period in pre post {
        use Dest* rel_los LTCDRG_Code using ${`period'_data}, clear
        if "`period'"=="pre"{
            merge m:1 LTCDRG_Code using ../temp/drgweights, ///
                assert(3) keep(3) keepusing(drgweight) nogen
        }
        drop LTCDRG_Code

        gen Dest = "Low" if (DestHomeCare | DestSNFIRF | DestLTCH | DestOther)
        replace Dest = "High" if (DestInpat | DestHospice)
        replace Dest = "Death" if missing(Dest)
        assert Dest == "Death" if DestDeath
        assert DestInpat | DestHospice | DestDeath if Dest!="Low"
        assert DestInpat | DestHospice if Dest=="High"
        assert Dest == "High" | Dest == "Low" | Dest == "Death"

        gen num = 1
        gen total = _N
        gen num_lo = Dest=="Low"
        gen num_hi = Dest=="High"
        gen num_death = Dest=="Death"

        count
        local `period'_ttl = `r(N)'
        count if rel_los<-15
        local `period'_left = `r(N)'
        count if rel_los>45
        local `period'_right = `r(N)'
        local frac_`period'_left = ``period'_left'/``period'_ttl'
        local frac_`period'_right = ``period'_right'/``period'_ttl'
        di "percent of `period'-period stays to left of tail is `frac_`period'_left'"
        di "percent of `period'-period stays to right of tail is `frac_`period'_right'"
        save ../temp/`period'_discharges, replace

        collapse (sum) num num_lo num_hi num_death (mean) total, by(rel_los)
        assert num == num_lo + num_hi + num_death
        gen `period'_share = num / total
        gen `period'_share_lo = num_lo / total
        gen `period'_share_hi = num_hi / total
        gen `period'_share_death = num_death / total
        sort rel_los
        gen cumulative = sum(num)
        gen total_remain = total - cumulative[_n-1] if _n > 1
        gen `period'_haz_rate = num / total_remain
        keep if rel_los>= -15 & rel_los <= 45
        save ../temp/`period'_los_dest, replace
    }
end

program discharges_weighted
    use rel_los num drgweight using ../temp/pre_discharges, clear
    replace num = num*drgweight
    egen total = total(num)

    collapse (sum) num (mean) total, by(rel_los)
    gen pre_share = num / total
    sort rel_los
    gen cumulative = sum(num)
    gen total_remain = total - cumulative[_n-1] if _n > 1
    gen pre_haz_rate = num / total_remain
    keep if rel_los>= -15 & rel_los <= 45
    save ../temp/pre_los_dest_wt, replace
end

program graph_discharges_by_los_dest
    use rel_los pre_share pre_share_lo pre_share_hi pre_share_death pre_haz_rate ///
        using ../temp/pre_los_dest, clear
    merge 1:1 rel_los using ../temp/post_los_dest, assert(3) keep(3) nogen ///
        keepusing(post_share post_share_lo post_share_hi post_share_death post_haz_rate)

    dschrg_dest_graph, pre(pre_share) post(post_share) ylab(0(0.03)0.12) ///
        ylin(0 0.03 0.06 0.09 0.12) yttl(Share of Stays Discharged) ///
        saving(../output/figures/hist_los_pre_post.pdf)
    dschrg_dest_graph, pre(pre_share_lo) post(post_share_lo) ylab(0(0.03)0.12) ///
        ylin(0 0.03 0.06 0.09 0.12) yttl(Share of Stays Discharged to Low-Acuity Dest's, size(small)) ///
        saving(../output/figures/hist_los_pre_post_dest_lo.pdf)
    dschrg_dest_graph, pre(pre_share_hi) post(post_share_hi) ylab(0(0.01)0.03) ///
        ylin(0 0.01 0.02 0.03) yttl(Share of Stays Discharged to High-Acuity Dest's, size(small)) ///
        saving(../output/figures/hist_los_pre_post_dest_hi.pdf)
    dschrg_dest_graph, pre(pre_share_death) post(post_share_death) ylab(0(0.01)0.03) ///
        ylin(0 0.01 0.02 0.03) yttl(Share of Stays Discharged to Death) ///
        saving(../output/figures/hist_los_pre_post_dest_death.pdf)

    export_excel_graph, x(rel_los) pre_y(pre_share) post_y(post_share) ///
        sav(hist_los_dest)
    export_excel_graph, x(rel_los) pre_y(pre_share_lo) post_y(post_share_lo) ///
        sav(hist_los_dest_lo)
    export_excel_graph, x(rel_los) pre_y(pre_share_hi) post_y(post_share_hi) ///
        sav(hist_los_dest_hi)
    export_excel_graph, x(rel_los) pre_y(pre_share_death) post_y(post_share_death) ///
        sav(hist_los_dest_death)
end

program graph_discharges_weighted

    use rel_los pre_share using ../temp/pre_los_dest_wt, clear
    merge 1:1 rel_los using ../temp/post_los_dest, assert(3) keep(3) nogen ///
        keepusing(post_share)
    dschrg_dest_graph, pre(pre_share) post(post_share) ylab(0(0.03)0.12) ///
        ylin(0 0.03 0.06 0.09 0.12) yttl(Share of Stays Discharged) ///
        saving(../output/figures/hist_los_pre_post_wt.pdf)
    export_excel_graph, x(rel_los) pre_y(pre_share) post_y(post_share) ///
        sav(hist_los_dest_wt)    
end

    program dschrg_dest_graph
        syntax, pre(string) post(string) ylab(string) ylin(string) yttl(string) saving(string)
        twoway (connected `pre' rel_los, msymbol(none) lcolor(gs8) lpattern(dash)) ///
            (connected `post' rel_los, msymbol(none) lcolor(black) lpattern(solid)), ///
            xlabel(#7, labsize(small)) ///
            ylabel(`ylab', labsize(small) format(%9.0gc) angle(0)) ///
            xtitle(Length of Stay Relative to SSO Threshold) ///
            ytitle(`yttl') ///
            yline(`ylin', lcolor(gs15)) ///
            legend(order(1 "Pre-PPS" 2 "PPS")) scheme(s1mono) 
        graph export `saving', replace
    end

    program export_excel_graph
        syntax, x(string) pre_y(string) post_y(string) sav(string)
        export excel `x' `pre_y' `post_y' using "../output/tables/discharge_rates_by_los", sheetreplace ///
            firstrow(variables) sheet(`sav') 
    end

* EXECUTE
main
