version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
preliminaries

global Flows="DestDeath DestHospice DestInpat DestLTCH DestSNF DestIRF DestHHA DestHome DestOther"

global pre_period_data = "../external/pre_pps/pre_pps_data"
global post_period_data = "../external/compiled/CompiledStayLevelData"


program main
    make_vars
    summary_stats
    export_to_excel
end


program make_vars
    foreach period in pre post {
        use medparid year dschrgcd dstntncd Dest* StillPatient using ${`period'_period_data}, clear
        drop DestSNFIRF
        gen DestSNF = dstntncd==3
        gen DestIRF = dstntncd==62
        drop DestHomeCare
        gen DestHHA = inlist(dstntncd, 6, 8)
        gen DestHome = dstntncd==1
        egen hasdstntn = rowtotal(DestHHA DestHome DestInpat DestLTCH DestSNF DestIRF ///
            DestHospice DestDeath DestOther StillPatient)
        assert hasdstntn == 1
        drop hasdstntn dschrgcd dstntncd
        save ../temp/`period'_period_LTCH, replace
    }
end

program summary_stats
    foreach period in pre post {
        use ../temp/`period'_period_LTCH, clear
        
        foreach dest in ${Flows} {
            qui sum `dest'
            matrix `period'_SumStats = ///
                nullmat(`period'_SumStats) \ r(mean)
        }
        qui count
        local N = `r(N)'
        matrix `period'_SumStats = ///
            nullmat(`period'_SumStats) \ 1 \ `N'
        
    }
    matrix granular_dests = pre_SumStats, post_SumStats
    matrix_to_txt, saving(../output/tables/granular_dests.txt) matrix(granular_dests) ///
        format(%20.4f) title(<tab:granular_dests>) replace
end

program export_to_excel
    matrix colnames granular_dests = Pre Post
    clear
    svmat granular_dests, names(col)
    gen Variable = ""
    replace Variable = "Discharged to Death" in 1
    replace Variable = "Discharged to Hospice" in 2
    replace Variable = "Discharged to Inpatient" in 3
    replace Variable = "Discharged to LTCH" in 4
    replace Variable = "Discharged to SNF" in 5
    replace Variable = "Discharged to IRF" in 6
    replace Variable = "Discharged to HHA" in 7
    replace Variable = "Discharged to Home" in 8
    replace Variable = "Discharged to Other Destination" in 9
    replace Variable = "Total" in 10
    replace Variable = "Number of Observations" in 11
    order Variable, first
    export excel using "../output/tables/granular_dests.xlsx", sheetreplace ///
        firstrow(variables) sheet(granular_dests) 
end

* EXECUTE
main
