version 14
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries

global min_day -15
global max_day 45
global yl1 "ylab(.002(.001).007)"
global yl30 "ylab(.04(.02).18)"
global test_bw = 3
global bstrap_its 200


set seed 1

program main
    graph_without_cis
    compute_graph_cis
    make_ci_graph
end

program graph_without_cis
    foreach n in 1 30 {
        foreach period in pre post  {
            use death_`n'day day ///
                using ../external/mort_haz/`period'_panel_pooled, clear
            collapse (mean) mort`n'_`period' = death_`n'day, by(day)
            save ../temp/mean_death_`n'day_`period', replace
        }
    }

    foreach n in 1 30 {
        use ../temp/mean_death_`n'day_pre, clear
        merge 1:1 day using ../temp/mean_death_`n'day_post, assert(3) keep(3) nogen

        twoway (line mort`n'_pre day, lpattern(solid) lcolor(gray)) ///
            (line mort`n'_post day, lpattern(solid) lcolor(black)) ///  
            , xtitle("Days relative to SSO threshold") ///
            ytitle("`n' Day Mortality") ${yl`n'} ///
            xline(0, lcolor(red)) ///
            xlab(${min_day}(5)${max_day}) ///
            scheme(s1mono) legend(order(2 "Pre-PPS" 3 "PPS"))
        graph export "../output/figures/mort_haz_`n'day.pdf", replace
        export excel using "../output/tables/mort_haz_rd_graphdata.xlsx", ///
            firstrow(variables) sheetreplace sheet(mort_haz_`n'day)
    }
end

program compute_graph_cis    
    foreach n in 1 30 {
        compute_lpoly, n(`n') i(0)        

        forval i = 1/${bstrap_its} {
            compute_lpoly, n(`n') i(`i') bsp  
        }
        
        use ../temp/lpoly_est_`n'day_1, clear
        forval i = 2/${bstrap_its} {
            merge 1:1 gridl gridr using ../temp/lpoly_est_`n'day_`i', assert(3) nogen 
        }
        save ../temp/merged_bootstraps_`n'day, replace

        foreach s in l r {
            use grid`s' mort`s'* using ../temp/merged_bootstraps_`n'day, clear
            egen lo_ci_`s' = rowpctile(mort`s'*), p(2.5)
            egen hi_ci_`s' = rowpctile(mort`s'*), p(97.5)
            rename grid`s' day
            drop mort`s'*
            save ../temp/ci_`n'day_`s'hs, replace

            use grid`s' mort`s' using ../temp/lpoly_est_`n'day_0, clear
            rename grid`s' day
            rename mort`s' pred_mort`s'
            save ../temp/lpoly_orig_`n'day_`s', replace
        }
    }
end

    program compute_lpoly
        syntax, n(str) i(str) [bsp]

        use death_`n'day day drgcd using ../external/mort_haz/post_panel_pooled, clear
        if "`bsp'" != "" {
            di("Bootstrapping")
            bsample, cluster(drgcd)
        }

        collapse (mean) death_`n'day, by(day)
        lpoly death_`n'day day if day < 0, kernel(rectangle) degree(1) bwidth($test_bw) nogr ///
            gen(gridl mortl`i')
        lpoly death_`n'day day if day >= 0, kernel(rectangle) degree(1) bwidth($test_bw) nogr ///
            gen(gridr mortr`i')
        keep gridl mortl gridr mortr
        drop if missing(gridl) & missing(gridr)
        save ../temp/lpoly_est_`n'day_`i', replace    
    end

program make_ci_graph
    foreach n in 1 30 {
        if "`n'" == "1" {
            local yl ylab(.002(.001).007)
        }
        if "`n'" == "30" {
            local yl  ylab(.04(.02).18)
        }

        use death_`n'day day ///
            using ../external/mort_haz/post_panel_pooled, clear
        collapse (mean) death_`n'day, by(day)
        save ../temp/mean_death_`n'day, replace
        merge 1:1 day using ../temp/lpoly_orig_`n'day_l, assert(1 2 3) keep(1 2 3) nogen keepusing(pred_mortl)
        merge 1:1 day using ../temp/lpoly_orig_`n'day_r, assert(1 2 3) keep(1 2 3) nogen keepusing(pred_mortr)
        merge 1:1 day using ../temp/ci_`n'day_lhs, assert(1 2 3) keep(1 2 3) nogen keepusing(lo_ci_l hi_ci_l)
        merge 1:1 day using ../temp/ci_`n'day_rhs, assert(1 2 3) keep(1 2 3) nogen keepusing(lo_ci_r hi_ci_r)

        twoway (scatter death_`n'day day) ///
            (line pred_mortl day if day < 0, lpattern(solid) lcolor(black)) ///
            (line pred_mortr day if day >= 0, lpattern(solid) lcolor(black)) ///
            (line lo_ci_l day if day < 0, lpattern(dash) lcolor(gray)) ///
            (line lo_ci_r day if day >= 0, lpattern(dash) lcolor(gray)) ///
            (line hi_ci_l day if day < 0, lpattern(dash) lcolor(gray)) ///   
            (line hi_ci_r day if day >= 0, lpattern(dash) lcolor(gray)) ///   
            , xtitle("Days relative to SSO threshold") ///
            ytitle("`n' Day Mortality") ${yl`n'} ///
            xline(0, lcolor(red)) ///
            xlab(${min_day}(5)${max_day}) ///
            scheme(s1mono) legend(order(2 "Best fit" 4 "95% CI (bootstrapped)"))
        graph export "../output/figures/mort_haz_rd_`n'day_cis.pdf", replace
        export excel using "../output/tables/mort_haz_rd_graphdata.xlsx", ///
            firstrow(variables) sheetreplace sheet(mort_haz_rd_`n'day_cis)
    }
end

* EXECUTE
main
