version 14
adopath + ../external/lib/stata/gslab_misc/ado 
adopath + ../external/lib/stata/ado 
preliminaries
global min_sso = -15
global max_sso = 45

program main
    make_moments_graphs
    post_dschrg_model_moments
    all_up_down
end

program make_moments_graphs
    foreach period in pre post {
        use if Dest !=. & inrange(rel_los, ${min_sso}, ${max_sso}) ///
            using ../external/post_ltch/post_ltch_`period'_period, clear
        gen N = 1
        collapse (mean) price_ub (sum) N, by(Dest rel_los)
        assert N>10
        drop N

        reshape wide price_ub, i(rel_los) j(Dest)
        rename price_ub1 price_ub_down
        rename price_ub2 price_ub_up

        post_ltch_graph, x(rel_los) down(price_ub_down) up(price_ub_up) ///
            yttl("Post-LTCH Total Medicare Payments") ylab(0(5000)45000) ///
            saving(../output/figures/post_ltch_drgprice_`period'_pps.pdf)

        export excel using "../output/tables/PostDischargePayments.xlsx", ///
            firstrow(variables) sheet(`period'_period) sheetreplace
    }
end

    program post_ltch_graph
        syntax, x(string) down(string) up(string) ylab(string) yttl(string) saving(string) 
        twoway (connected `down' `x', msymbol(none) lcolor(gs8) lpattern(solid)) ///
            (connected `up' `x', msymbol(none) lcolor(black) lpattern(solid)), ///
            xlabel(${min_sso}(5)${max_sso}, labsize(small)) ///
            xtitle(Length of Stay Relative to SSO Threshold) /// 
            ylabel(`ylab', angle(0) labsize(small)) ytitle(`yttl') ///
            scheme(s1mono) legend(order(1 "Downstream" 2 "Upstream"))
        graph export `saving', replace
    end

program post_dschrg_model_moments
    * NOTE: We use upper bound because it is used in the summary table,
    * but in previous versions we may have used lower bound.
    * Please confirm which one to use.
    foreach period in pre post {
        use price_ub los_ub mort60 Dest rel_los ///
            if Dest != . & inrange(rel_los, ${min_sso}, ${max_sso}) ///
            using ../external/post_ltch/post_ltch_`period'_period, clear
        gen N = 1
        collapse (mean) price_ub los_ub mort60 (sum) N, by(Dest rel_los)
        assert N > 10
        drop N
        export excel using "../output/tables/post_dschrg_stats.xlsx", ///
            sheetreplace firstrow(variables) sheet(stats_by_los_`period')

        use price_ub los_ub mort60 Dest rel_los ///
            if Dest != . & inrange(rel_los, ${min_sso}, ${max_sso}) ///
            using ../external/post_ltch/post_ltch_`period'_period, clear
        collapse (mean) price_ub los_ub mort60, by(Dest)
        export excel using "../output/tables/post_dschrg_stats.xlsx", ///
            sheetreplace firstrow(variables) sheet(pooled_n15_45_`period')
    }
end

program all_up_down
    foreach period in pre post {
        use if Dest !=. using ../external/post_ltch/post_ltch_`period'_period, clear
        count
        local N_all = `r(N)'
        count if Dest==1
        local N_down = `r(N)'
        count if Dest==2
        local N_up = `r(N)'
        matrix `period'_all_up_down = nullmat(`period'_all_up_down) \ (`N_all', `N_up', `N_down')
        foreach var in mort30 mort90 price_ub price_lb los_ub los_lb {
            sum `var'
            local all = `r(mean)'
            sum `var' if Dest==1
            local down = `r(mean)'
            sum `var' if Dest==2
            local up = `r(mean)'
            matrix `period'_all_up_down = nullmat(`period'_all_up_down) \ (`all', `up', `down')
        }        
    }
    matrix all_up_down = pre_all_up_down ,  post_all_up_down
    matrix_to_txt, saving(../output/tables/all_up_down.txt) matrix(all_up_down) ///
        format(%20.4f) title(<tab:all_up_down>) replace
    matrix colnames all_up_down = Overall_pre Upstream_pre Downstream_pre Overall_post Upstream_post Downstream_post
    clear
    svmat all_up_down, names(col)
    gen Variable = ""
    replace Variable = "N" in 1
    replace Variable = "30-Day Post-Discharge Mortality" in 2
    replace Variable = "90-Day Post-Discharge Mortality" in 3
    replace Variable = "Payments (baseline)" in 4
    replace Variable = "Payments (LB)" in 5
    replace Variable = "Facility Days (baseline)" in 6
    replace Variable = "Facility Days (LB)" in 7

    order Variable, first
    export excel using "../output/tables/PostDischargePanel.xlsx", ///
        firstrow(variables) sheet(all_up_down) replace
end

* EXECUTE
main
