version 12
adopath + ../external/lib/stata/gslab_misc/ado 
adopath + ../external/lib/stata/ado 
preliminaries
set more off

program main
    load_data
    code_names
    top_prcdr
    proc_stats
    export_tables
end

program load_data
    
    use medparid year prcdrseq bene_id prcdrcd prcdrdt using ///
        ../external_links/proc/proc_append_pre_ltch, clear
    merge m:1 medparid year using ../external/pre_ltch/pre_pps_data, ///
        assert(1 2 3) keep(2 3) nogen // keep the baseline sample; not everyone had procedures
    save ../temp/pre_LTCH_loaded, replace

    use medparid year prcdrseq bene_id prcdrcd prcdrdt using ///
        ../external_links/proc/proc_append_pre_ach, clear
    merge m:1 medparid year using ../external_links/ach/pre_pps_ACH, ///
        assert(1 2 3) keep(2 3) nogen
    save ../temp/pre_ACH_loaded, replace

    use medparid year prcdrseq bene_id prcdrcd prcdrdt using ///
        ../external_links/proc/proc_append_pre_snf_irf, clear
    merge m:1 medparid year using ../external/snf/pre_pps_SNF_IRF, ///
        assert(1 2 3) keep(2 3) nogen
    save ../temp/pre_SNF_IRF_loaded, replace

    use medparid year prcdrseq bene_id prcdrcd prcdrdt using ///
        ../external_links/proc/proc_append_post_ltch, clear
    merge m:1 medparid year using ../external/post_ltch/CompiledStayLevelData, ///
        assert(1 2 3) keep(2 3) nogen
    save ../temp/post_LTCH_loaded, replace

    use medparid year prcdrseq bene_id prcdrcd prcdrdt using ///
        ../external_links/proc/proc_append_post_ach, clear
    merge m:1 medparid year using ../external_links/ach/post_pps_ACH, ///
        assert(1 2 3) keep(2 3) nogen
    save ../temp/post_ACH_loaded, replace

    use medparid year prcdrseq bene_id prcdrcd prcdrdt using ///
        ../external_links/proc/proc_append_post_snf_irf, clear
    merge m:1 medparid year using ../external/snf/post_pps_SNF_IRF, ///
        assert(1 2 3) keep(2 3) nogen
    save ../temp/post_SNF_IRF_loaded, replace

end

program code_names
    import excel ../external/proc/proc_codes_fy10.xls, firstrow clear
    rename LONGDESCRIPTION name
    rename PROCEDURECODE prcdrcd
    keep prcdrcd name
    isid prcdrcd
    save ../temp/proc_code_names, replace
end

program top_prcdr
    foreach period in pre post {
        foreach dataset in LTCH ACH SNF_IRF {
            use ../temp/`period'_`dataset'_loaded, clear
            replace prcdrcd = "None" if prcdrcd == ""  
            * Drop if procedure date comes after the discharge date or before admsn date - should be errors
            drop if (prcdrdt > dschrgdt & prcdrdt !=.) | prcdrdt < admsndt

            egen stay_proc_tag = tag(medparid year prcdrcd)
            egen stay_tag = tag(medparid year)
            egen total_stays = total(stay_tag)
            collapse (sum) num_stays=stay_proc_tag (mean) total_stays, by(prcdrcd)
            gen share_stays = num_stays/total_stays
            gsort -share_stays
            drop if prcdrcd == "None"
            keep if _n <= 3
            save ../temp/TopProcedures_`period'_`dataset', replace
        }
    }
end
        
* Stats on procedures in general
program proc_stats
    foreach period in pre post {
        foreach dataset in LTCH ACH SNF_IRF {
            use ../temp/`period'_`dataset'_loaded, clear
            * Drop if procedure date comes after the discharge date or before admsn date - should be errors
            drop if (prcdrdt > dschrgdt & prcdrdt !=.) | prcdrdt < admsndt

            * Count number of procedures
            bys medparid year: egen num_proc = max(prcdrseq)
            replace num_proc = 0 if prcdrseq==.
            collapse (max) num_proc, by(medparid year)

            * Stays without any procedures
            gen zero_proc = num_proc == 0

            * Average number of procedures conditional having any
            gen num_gvn_any = num_proc if num_proc != 0

            foreach var in zero_proc num_gvn_any {
                sum `var' if ~missing(`var')
                mat proc_`period'_`dataset' = nullmat(proc_`period'_`dataset') \ r(mean)
            }
            matrix colnames proc_`period'_`dataset' = outcome
            clear
            svmat proc_`period'_`dataset', names(col)
            gen name = ""
            replace name = "Fraction with no procedures" in 1
            replace name = "Average number of procedures conditional on having any" in 2
            save ../temp/proc_stats_`period'_`dataset', replace
        }
    }
end
    
program export_tables
    foreach period in pre post {
        foreach dataset in LTCH ACH SNF_IRF {
            use ../temp/TopProcedures_`period'_`dataset', clear
            merge 1:1 prcdrcd using ../temp/proc_code_names, assert(2 3) keep(3) ///
                keepusing(name) nogen
            gsort -share_stays
            rename share_stays outcome
            keep name outcome
            save ../temp/ForMatx, replace

            use ../temp/proc_stats_`period'_`dataset', clear
            append using ../temp/ForMatx
            order name outcome
            gen row = _n
            gen per = "`period'"
            gen dta = "`dataset'"
            save ../temp/paper_info_`period'_`dataset', replace      
        }
    }

    clear
    foreach period in pre post {
        foreach dataset in LTCH ACH SNF_IRF {
            append using ../temp/paper_info_`period'_`dataset'
        }
    }
    rename name name_
    rename outcome outcome_
    reshape wide name_ outcome_, i(row dta) j(per) string
    foreach v in name_pre name_post outcome_pre outcome_post {
        rename `v' `v'_
    }
    reshape wide name* outcome*, i(row) j(dta) string
    order name_pre_ACH outcome_pre_ACH name_pre_LTCH outcome_pre_LTCH ///
        name_pre_SNF_IRF outcome_pre_SNF_IRF name_post_ACH outcome_post_ACH ///
        name_post_LTCH outcome_post_LTCH name_post_SNF_IRF outcome_post_SNF_IRF
    drop row
    export excel using "../output/tables/paper_proc.xlsx", sheetreplace ///
        firstrow(variables)
    drop name*
    dta_to_txt, saving(../output/tables/paper_proc.txt) title(<tab:paper_proc>) ///
        replace noquote nonames
end

** EXECUTE
main
