version 12
adopath + ../external/lib/stata/gslab_misc/ado 
adopath + ../external/lib/stata/ado 
preliminaries

global in_vars = "medparid bene_id src_adms ms_cd year age_cnt sex race mort30day_dschrg mort60day_admsn mort90day_admsn mcaid util_day drgprice Orig* Dest* StillPatient dstntncd dschrgcd prvdrnum cvrchrg coin_amt ded_amt blddedam deathdt_since_admsn LTCDRG_Code"

global Attributes="age_cnt male race_white race_black oldagequalify mcaid"
global Health="num_CC mort30day_admsn mort90day_admsn home_90days util_day"
global Money="drgprice benecost totalcost"
global Flows="DestHomeCare DestInpat DestLTCH DestSNFIRF DestHospice DestDeath DestOther StillPatient OrigReferral OrigER OrigACH OrigLTCH OrigSNF OrigIRF OrigOtherHosp OrigOther"

program main
    load_data
    make_vars
    summary_stats
    load_drgs
    topDRGs
    export_to_excel
end

program load_data
    use ${in_vars} using ../external/pre_pps/pre_pps_data, clear
    gen calendar_year=year
    merge m:1 bene_id calendar_year using ../external/CC/CC_append, assert(1 2 3) keep(1 3) nogen
    rename LTCDRG_Code drg_cd
    gen totalcost = drgprice
    gen costperday = drgprice/util_day
    save ../temp/pre_period_LTCH_loaded, replace

    use totalcost costperday ${in_vars} using ../external/compiled/CompiledStayLevelData, clear
    gen calendar_year=year
    merge m:1 bene_id calendar_year using ../external/CC/CC_append, assert(1 2 3) keep(1 3) nogen
    rename LTCDRG_Code drg_cd
    save ../temp/post_period_LTCH_loaded, replace
    
    foreach period in pre post {
        use ../external_links/ACH/`period'_pps_ACH, clear
        gen mort30day_dschrg=deathdt_since_dschrg <= 30
        gen totalcost=.
        gen costperday=.
        save ../temp/`period'_period_ACH_loaded, replace
    }

    foreach period in pre post {
        use ../external/snf/`period'_pps_SNF_IRF, clear
        gen mort30day_dschrg=deathdt_since_dschrg <= 30
        gen totalcost=.
        gen costperday=.
        destring sex race, replace
        save ../temp/`period'_period_SNF_IRF_loaded, replace
    }
end

program make_vars
    foreach period in pre post {
        foreach dataset in ACH LTCH SNF_IRF {
            use ../temp/`period'_period_`dataset'_loaded, clear
            egen hasdstntn = rowtotal(DestHomeCare DestInpat DestLTCH DestSNFIRF ///
                DestHospice DestDeath DestOther StillPatient)
            assert hasdstntn == 1
            drop hasdstntn
            gen male=sex==1
            gen race_white=race==1
            gen race_black=race==2
            gen race_other=inlist(race, 3,4,5,6)
            gen benecost = coin_amt + ded_amt + blddedam
            gen benecost_perday = benecost/util_day
            gen mort30day_admsn = deathdt_since_admsn<=30

            gen oldagequalify = inlist(ms_cd,10,11)
            gen payperday = drgprice/util_day

            merge m:1 year using ../external/cpi/cpi_annual, assert(2 3) keep(3) nogen
            foreach var in drgprice payperday benecost benecost_perday totalcost costperday {
                replace `var' = `var'/pct_of_12
            }  

            merge 1:1 medparid year using ///
                 ../external_links/home_90days/home_90days_`period'_`dataset', ///
                assert(2 3) keep(3) nogen keepusing(home_90days)
            save ../temp/`period'_period_`dataset', replace
        }    
        use ../temp/`period'_period_LTCH, clear
        append using ../temp/`period'_period_SNF_IRF
        save ../temp/`period'_period_PAC, replace
    }
end


program summary_stats
    foreach period in pre post {
        foreach dataset in ACH LTCH SNF_IRF PAC {
            use ../temp/`period'_period_`dataset', clear
            foreach var in Attributes Health Flows {
                foreach outcome in ${`var'} {
                    qui sum `outcome'
                    matrix `period'_SumStats_`dataset'_`var' = ///
                        nullmat(`period'_SumStats_`dataset'_`var') \ r(mean)
                }
                qui count
                local N = `r(N)'
                matrix `period'_SumStats_`dataset'_`var' = ///
                    nullmat(`period'_SumStats_`dataset'_`var') \ `N'
            }

            ** Money goes separately because we take ratio of averages 
                * (not average of ratios)
            sum util_day
            local avg_los = r(mean)
            foreach outcome in $Money {
                qui sum `outcome'
                local out=r(mean)
                local outperday = `out' / `avg_los'
                matrix both = `out' \ `outperday'
                matrix `period'_SumStats_`dataset'_Money = ///
                    nullmat(`period'_SumStats_`dataset'_Money) \ both
            }
            qui count
            local N = `r(N)'
            matrix `period'_SumStats_`dataset'_Money = ///
                nullmat(`period'_SumStats_`dataset'_Money) \ `N'
        }

        foreach var in Attributes Health Money Flows {
            mat `period'_SumStats_all_`var' =  `period'_SumStats_ACH_`var'[1...,1...], ///
                `period'_SumStats_LTCH_`var'[1...,1...], ///
                `period'_SumStats_SNF_IRF_`var'[1...,1...], ///
                `period'_SumStats_PAC_`var'[1...,1...] ///

        }
    }

    * for spreadsheet:
    foreach pd in pre post {
        mat `pd'_SumStats_all =  `pd'_SumStats_all_Attributes[1..6, 1...] \ ///
                                `pd'_SumStats_all_Health[1..5, 1...] \ ///
                                `pd'_SumStats_all_Money[1..6, 1...] \ ///
                                `pd'_SumStats_all_Flows[1..., 1...]
    }

    * for lyx:
    foreach var in Attributes Health Money Flows {
        mat SumStats_all_`var' = pre_SumStats_all_`var', post_SumStats_all_`var'
        matrix_to_txt, saving(../output/tables/SumStats_all_`var'.txt) ///
            matrix(SumStats_all_`var') format(%20.4f) ///
            title(<tab:SumStats_all_`var'>) replace
    }
end

program load_drgs
    insheet using  "../external/DRG_codes/FY 2010 FR Table 5.txt", clear
    keep msdrg msdrgtitle
    * Dropping rows containing notes on the data
    drop if _n >= 747
    destring msdrg, replace
    rename msdrg drg_cd
    rename msdrgtitle drgtitle
    save ../temp/drg_codes, replace    
end

program topDRGs
    * Can't match DRG codes for pre-period because they changed with the PPS system
    * (LTCDRG_Code/drg_cd is the MS-LTC-DRG code used in the PPS system)

    foreach dataset in ACH LTCH SNF_IRF PAC {
        use drg_cd using ../temp/post_period_`dataset', clear
        gen num=1
        gen N=_N
        collapse (sum) drg_size=num (mean) N, by(drg_cd)
        merge 1:1 drg_cd using ../temp/drg_codes, assert(1 2 3) keep(1 3) ///
            keepusing(drgtitle)
        qui count if _merge==1
        di "Number of unnamed DRGs in post period is `r(N)'"
        drop _merge
        gen Share=drg_size/N
        gsort -Share
        local drg25th_size=drg_size[25]
        local drg50th_size=drg_size[50]
        gen Cumul=sum(Share)
        gen rank=_n
        keep if (rank>=1 & rank<=3)
        keep rank drg_cd drgtitle Share
        rename drg_cd drg_cd_post_`dataset'
        rename drgtitle drgtitle_post_`dataset'
        rename Share Share_post_`dataset'
        save ../temp/post_`dataset'_TopDRGs, replace
    }
    
    clear
    foreach dataset in ACH LTCH SNF_IRF PAC {
        append using ../temp/post_`dataset'_TopDRGs
    }
    collapse (firstnm) drg_cd* drgtitle* Share*, by(rank)
    order rank *_ACH *_LTCH *_SNF_IRF *_PAC, first

    mkmat  Share_post_ACH Share_post_LTCH Share_post_SNF_IRF Share_post_PAC, ///
        matrix(post_TopDRGs_frac)
    matrix post_TopDRGs_frac = (., ., ., . \ ., ., ., . \ ., ., ., .) , post_TopDRGs_frac
    matrix SumStats_all_Health_DRGs =  ///
        SumStats_all_Health[1..4, 1...] \ ///
        post_TopDRGs_frac \ ///
        SumStats_all_Health[5..6, 1...]
    matrix_to_txt, saving(../output/tables/SumStats_all_Health_DRGs.txt) ///
        matrix(SumStats_all_Health_DRGs) format(%20.4f) title(<tab:SumStats_all_Health_DRGs>) ///
        replace

    foreach dataset in ACH LTCH SNF_IRF PAC {
        rename drg_cd_post_`dataset' `dataset'_code
        rename Share_post_`dataset' `dataset'_share
        rename drgtitle_post_`dataset' `dataset'_drgtitle
    }
    export excel using "../output/tables/SummaryStats.xlsx", ///
        sheetreplace firstrow(variables) sheet(post_TopDRGs)
end

program export_to_excel
    foreach period in pre post {
        matrix colnames `period'_SumStats_all = ACH LTCH SNF_IRF PAC
        clear
        svmat `period'_SumStats_all, names(col)
        gen Variable = ""
        replace Variable = "Age" in 1
        replace Variable = "Male" in 2
        replace Variable = "White" in 3
        replace Variable = "Black" in 4
        replace Variable = "Qualified for Medicare Through Old Age" in 5
        replace Variable = "Also on Medicaid" in 6
        replace Variable = "Number of Chronic Conditions" in 7
        replace Variable = "30 Day Mortality Since Admission" in 8
        replace Variable = "90 Day Mortality Since Admission" in 9
        replace Variable = "Home After 90 Days" in 10
        replace Variable = "LOS" in 11
        replace Variable = "Total Medicare Payments for Stay" in 12
        replace Variable = "Medicare Payments per Day" in 13
        replace Variable = "Total Out-of-Pocket Payments for Stay" in 14
        replace Variable = "Out-of-Pocket Payments per Day" in 15
        replace Variable = "Total Reported Costs" in 16
        replace Variable = "Reported Costs per Day" in 17
        replace Variable = "Discharged to Home/HomeHealth" in 18
        replace Variable = "Discharged to Inpatient" in 19
        replace Variable = "Discharged to LTCH" in 20
        replace Variable = "Discharged to SNF/IRF" in 21
        replace Variable = "Discharged to Hospice" in 22
        replace Variable = "Discharged to Death" in 23
        replace Variable = "Discharged to Other Destination" in 24
        replace Variable = "Discharge Destination Missing" in 25
        replace Variable = "Admission by Referral" in 26
        replace Variable = "Admission from ER" in 27
        replace Variable = "Admission by Transfer from ACH" in 28
        replace Variable = "Admission by Transfer from LTCH" in 29    
        replace Variable = "Admission by Transfer from SNF" in 30
        replace Variable = "Admission by Transfer from IRF" in 31
        replace Variable = "Admission by Transfer from Other Hospital" in 32
        replace Variable = "Admission by Other" in 33
        replace Variable = "Number of Observations" in 34
        order Variable, first
        export excel using "../output/tables/SummaryStats.xlsx", sheetreplace ///
            firstrow(variables) sheet(`period'_SumStats_all)
    }
end

*Execute
main
