version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries, maxvar(30000) matsize(11000)

global medparpay="drgprice"
global cgspay="cgs_drgprice"

program main
    load_cgs_output
    gen_variables
    calculate_jump
    gen_compiled_data
end

program load_cgs_output
    use "../external_links/CGS Calculate/cgs_output", clear	
    gen medpartotalpayment=drgprice+outlramt
    gen cgs_drgprice=cgs_totalpayment-cgs_costoutlier
    replace rel_los=cgs_lengthofstay-Kink if fakedata==1
    assert rel_los==cgs_lengthofstay-Kink if fakedata==0
    gen totalcost=cvrchrg*costtochargeratio
    gen costperday=chargesperday*costtochargeratio
    save ../temp/cgs_output, replace
end

program gen_variables
    use if fakedata==0 using ../temp/cgs_output, clear
    bys LTCDRG_Code prvdrnum rate_year: egen cellsize_DHR=count(rate_year)
    * The reverse of the following assert is no longer true due to CGS related drops
    assert cellsize_DHR==1 if LOOMchargesperday==. 

    gen mort30day_dschrg=deathdt_since_dschrg<=30

    * Acct may need to be revised - now we are ignoring any outlier payment
    gen acctprofit=(${cgspay}-totalcost)/1000
    gen abslos=abs(rel_los)
    forv x=-10/10 {
        local i=`x'+11
        gen los_ind`i'=(rel_los<=`x')
    }
    save ../temp/original_data, replace
end

program calculate_jump
    use if ${cgspay}!=. using ../temp/cgs_output, clear
    assert rel_los==cgs_lengthofstay-Kink
    keep if inlist(rel_los,-1,0)
    sort medparid rel_los
    gen calc_jump=${cgspay}-${cgspay}[_n-1] if medparid==medparid[_n-1]
    collapse (mean) calc_jump, by(medparid prvdrnum rate_year LTCDRG_Code)
    replace calc_jump = calc_jump/1000
    * Drop negative jumps for now; these should be errors
    drop if calc_jump <0
    save ../temp/calc_jump, replace
end

program gen_compiled_data
    use ../temp/original_data, clear
    merge 1:1 medparid using ../temp/calc_jump, assert(1 2 3) keep(3) nogen
    merge m:1 prvdrnum year using ../external/POS/POS_output, assert(1 2 3) keep(1 3) nogen
    rename cntrl* POScntrl*
    gen fyear=rate_year
    replace fyear=2008 if year(dschrgdt)==2008 & month(dschrgdt)<=9
    merge m:1 prvdrnum fyear using ../external/HCRIS/hcris_ltch, assert(1 2 3) keep(1 3) nogen
    rename cntrl* HCRIScntrl*
    gen aha_data_year=rate_year
    merge m:1 prvdrnum aha_data_year using ../external/AHA/aha_append_ltch, ///
        assert(1 2 3) keep(1 3) nogen keepusing(cntrl*)        
    rename cntrl* AHAcntrl*

    * Restrict sample to end on July 31, 2012 to deal with truncation issues 
    *   for 90-day mortality and (~150-day spell) post-LTCH discharges
    keep if (year(dschrgdt) > 2007 & year(dschrgdt) < 2012) | ///
            (year(dschrgdt) == 2007 & month(dschrgdt) >= 10) | ///
            (year(dschrgdt) == 2012 & month(dschrgdt) <= 7)

    * Merge origins
    merge 1:1 medparid year using ../external_links/origs/orig_from_spells_post, ///
        assert(2 3) keep(3) keepusing(Orig*) nogen

    label variable calc_jump "Calculated jump in thousand dollars"
    label variable costperday "CostperDay=TotalCost/LOS"
    label variable acctprofit "Accounting Profit in thousand dollars"
    label variable mort30day_dschrg "30 day mortality since discharge"
    label variable mort60day_admsn "60 day mortality since admission"
    label variable mort90day_admsn "90 day mortality since admission" 
    label variable cellsize_DHR "Number of stays in DRG-Hospital-RY"
    save_data ../output/CompiledStayLevelData, key(medparid year) replace
end

*Execute
main
