version 12
adopath + ../external/lib/stata/gslab_misc/ado 

preliminaries

program main
    clean_diag_proc_files, keep_condition(year>=2000 & year<=2002)
    prepare_input using ../external/medpar_samp/medpar_sample_pre, ///
        diag_file(../temp/diag) proc_file(../temp/proc)
    gen_csv_input_files, date_range(year(dschrgdt) == 2000 & month(dschrgdt) <= 9) ///
        grouper_version(17) 
    gen_csv_input_files, date_range((year(dschrgdt) == 2000 & month(dschrgdt) >= 10) | ///
        (year(dschrgdt) == 2001 & month(dschrgdt) <= 9)) ///
        grouper_version(18) 
    gen_csv_input_files, date_range((year(dschrgdt) == 2001 & month(dschrgdt) >= 10) | ///
        year(dschrgdt) == 2002 & month(dschrgdt) <= 9) ///
        grouper_version(19) 
end

program clean_diag_proc_files
    syntax, keep_condition(str)

    use medparid year dgnscd dgnscdseq using ../external_links/prc_dgn/diag_append_pre_ltch, clear
    keep if `keep_condition'
    reshape wide dgnscd, i(medparid year) j(dgnscdseq)
    rename dgnscd1 prmdgnscd
    egen dgnscd2_25=concat(dgnscd*), punct(";")
    keep medparid year prmdgnscd dgnscd2_25
    save ../temp/diag, replace

    use medparid year prcdrseq prcdrcd using ../external_links/prc_dgn/proc_append_pre_ltch, clear
    keep if `keep_condition'
    reshape wide prcdrcd, i(medparid year) j(prcdrseq)    
    egen prcdrcd=concat(prcdrcd*), punct(";")
    keep medparid year prcdrcd
    save ../temp/proc, replace
end

program prepare_input
    syntax using, diag_file(str) proc_file(str)

    use `using', clear
    assert year(dschrgdt)>=2000 & (year(dschrgdt) < 2002 | (year(dschrgdt) == 2002 & month(dschrgdt) <= 9))
    merge 1:1 medparid year using `diag_file', assert(1 2 3) keep(3) nogen
    merge 1:1 medparid year using `proc_file', assert(1 2 3) keep(1 3) nogen

    rename drg_cd old_drg
    gen cgs_id = _n
    gen disablehac = 1 //LTCH are exempt from HAC-POA payment provision
    save ../temp/cgs_input, replace
end

program gen_csv_input_files
    syntax, date_range(string) grouper_version(integer)

    use if `date_range' using ../temp/cgs_input, clear
    keep cgs_id prmdgnscd dgnscd2_25 prcdrcd dstntncd age_cnt sex disablehac
    order cgs_id prmdgnscd dgnscd2_25 prcdrcd dstntncd age_cnt sex disablehac
    export delimited using ../temp/input_v`grouper_version', replace
end

*Execute
main
