version 12
adopath + ../external/lib/stata/gslab_misc/ado 
preliminaries

program main
    load_sso_thres
    compile_cgs_output
end

program load_sso_thres
    * Assign FY 2008 SSO Threshold to pre-PPS stays
    use ../external/Kink/Kink, clear
    keep if fyear == 2008
    save ../temp/Kink, replace
end

program compile_cgs_output
    foreach file in v17 v18 v19 {
        insheet using  ../temp/output_`file'.csv, clear
        save ../temp/output_`file', replace
    }
    use ../temp/output_v17, clear
    append using ../temp/output_v18
    append using ../temp/output_v19
    rename accountnumber cgs_id
    merge 1:1 cgs_id using ../temp/cgs_input, assert(3) keep(3) nogen
    assert principaldiagnosis == prmdgnscd

    rename drg LTCDRG_Code
    label variable LTCDRG_Code "MS-DRG code obtained using grouper version 26.0"
    label variable old_drg "Original CMS-DRG code"
    drop principaldiagnosis cgs_id

    * In the PPS data we merge with AHA by PPS rate years
    * Rate years do not exist pre-PPS, we use data reporting years for now
    gen aha_data_year = year
    merge m:1 prvdrnum aha_data_year using ../external/AHA/aha_append, ///
        assert(1 2 3) keep(1 3) nogen keepusing(cntrl*)
    rename cntrl* AHAcntrl*

    drop ach ltch snf irf    
    merge m:1 LTCDRG_Code using ../temp/Kink, assert(2 3) keep(3) keepusing(Kink) nogen
    gen rel_los = util_day - Kink

    merge 1:1 medparid year using ../external_links/origs/orig_from_spells_pre, ///
        assert(2 3) keep(3) keepusing(Orig*) nogen

    gen mort30day_dschrg=deathdt_since_dschrg <= 30

    save_data ../output/pre_pps_data, key(medparid year) replace
end

* Execute
main
