version 12
adopath + ../external/lib/stata/gslab_misc/ado 

preliminaries

program main
    clean_diag_proc_files, keep_condition(year>=2007 & year <= 2012)
    gen_input_original using ../external/medpar_samp/medpar_sample_post, ///
        diag_file(../temp/diag) proc_file(../temp/proc) ///
        original_data(../output_local/original_data) sso_file(../external/parameter/Kink)
    gen_input_counterfactual, original_data(../output_local/original_data) num_input_files(20)
end

program clean_diag_proc_files
    syntax, keep_condition(str)
    use medparid year dgnscd dgnscdseq using ../external_links/prc_dgn/diag_append_post_ltch, clear
    keep if `keep_condition'
    reshape wide dgnscd, i(medparid year) j(dgnscdseq)
    rename dgnscd1 prmdgnscd
    egen dgnscd2_25=concat(dgnscd*), punct(";")
    keep medparid year prmdgnscd dgnscd2_25
    save ../temp/diag, replace

    use medparid year prcdrseq prcdrcd using ../external_links/prc_dgn/proc_append_post_ltch, clear
    keep if `keep_condition'
    reshape wide prcdrcd, i(medparid year) j(prcdrseq)    
    egen prcdrcd=concat(prcdrcd*), punct(";")
    keep medparid year prcdrcd
    save ../temp/proc, replace
end

program gen_input_original
    syntax using, diag_file(str) proc_file(str) original_data(str) sso_file(str) ///

    use `using', clear
    assert year(dschrgdt) > 2007 | (year(dschrgdt) == 2007 & month(dschrgdt) >= 10)
    merge 1:1 medparid year using `diag_file', assert(1 2 3) keep(3) nogen
    merge 1:1 medparid year using `proc_file', assert(1 2 3) keep(1 3) nogen
    rename drg_cd LTCDRG_Code
    gen rate_year = year(dschrgdt) if (year(dschrgdt) >= 2009 & month(dschrgdt) <= 9) | ///
        (year(dschrgdt) <= 2008 & month(dschrgdt) <= 6)
    replace rate_year = year(dschrgdt) + 1 if rate_year == .
    
    * fyear generated following the dates of the schedules changes
    gen fyear = rate_year if rate_year >= 2010
    replace fyear = 2008 if (year(dschrgdt) == 2007 & month(dschrgdt) >= 10) | ///
        (year(dschrgdt) == 2008 & month(dschrgdt) < 10)
    replace fyear=2009 if (year(dschrgdt) == 2008 & month(dschrgdt) >= 10)| ///
        (year(dschrgdt) == 2009 & month(dschrgdt) < 6) | ///
        (year(dschrgdt) == 2009 & month(dschrgdt) == 6 & day(dschrgdt) <= 2)
    replace fyear = 2009.5 if ///
        (year(dschrgdt) == 2009 & month(dschrgdt) < 10 & month(dschrgdt) > 6) | ///
        (year(dschrgdt) == 2009 & month(dschrgdt) == 6 & day(dschrgdt) >= 3)
    assert fyear == 2009 if rate_year == 2009 & fyear != 2008 & fyear != 2009.5
    assert rate_year == 2009 if inlist(fyear,2009,2009.5)
    assert fyear == 2008 & rate_year == 2009 if year(dschrgdt) == 2008 & month(dschrgdt) < 10 ///
        & month(dschrgdt) >= 7

    merge m:1 LTCDRG_Code fyear using `sso_file', assert(2 3) keep(3) nogen
    drop fyear
    gen rel_los = util_day - Kink
    gen id = _n
    gen chargesperday = cvrchrg/util_day
    bys LTCDRG_Code prvdrnum rate_year: egen num_in_cell = count(id)
    bys LTCDRG_Code prvdrnum rate_year: egen sum_charges = total(chargesperday)
    gen LOOMchargesperday = (sum_charges-chargesperday)/(num_in_cell-1)
    save_data `original_data', replace key(medparid year) log("../output/data_file_manifest.log")
    
    keep if util_day > (Kink+1)
    gen util_day_cf = util_day
    gen rel_los_cf = rel_los
    save ../temp/original_los2plus, replace
end

program gen_input_counterfactual    
    syntax, original_data(str) num_input_files(integer)

    use `original_data', clear
    gen num_expand = Kink + 1
    expand num_expand
    bys id: gen rel_los_cf = 1 if _n == 1
    replace rel_los_cf = rel_los_cf[_n-1] - 1 if id == id[_n-1]
    gen util_day_cf = Kink + rel_los_cf
    assert util_day >= 1
    replace dschrgdt = admsndt + util_day_cf
    append using ../temp/original_los2plus
    gen fakedata = rel_los != rel_los_cf
    assert util_day == util_day_cf if fakedata == 0
    assert util_day != util_day_cf if fakedata == 1  
    foreach event in admsn dschrg {
    	foreach time in month day {
            gen `event'`time' = string(`time'(`event'dt),"%02.0f")
        }
        gen `event'year = string(year(`event'dt),"%04.0f")
        egen `event'date = concat(`event'month `event'day `event'year)
    }
    gen cgs_id = _n
    gen schedule = "9952"
    gen disablehac = 1 //LTCH are exempt from HAC-POA payment provision
    count
    local num_obs = r(N)
    * Assume charges are per day charge times length of stay
    replace cvrchrg = round(chargesperday*util_day_cf) if fakedata == 1
    * The assert should be 0 theoretically but due to rounding is just very close to 0
    assert abs(chargesperday*util_day_cf - cvrchrg) < 0.5 if fakedata == 0
    save ../temp/cgs_input, replace

    * Assume charges are leave-one-out mean charges per day times length of stay
    replace cvrchrg=round(LOOMchargesperday*util_day_cf) //note all obs are changed regardless of fakedata
    save ../temp/cgs_input_LOOM, replace

    keep cgs_id id fakedata
    save_data ../output_local/cgs_id_xwalk, replace key(cgs_id) ///
        log("../output/data_file_manifest.log")

    foreach input in cgs_input cgs_input_LOOM {
        forv i = 1/`num_input_files' {
            if `i' == 1 {
                local start = 0
            }
            else {
                local start = (`i' - 1)*(`num_obs'/`num_input_files')
            }
            if `i' == `num_input_files' {
                local end = `num_obs' + 1
            }
            else {
            local end = `i'*(`num_obs'/`num_input_files')
            }
            use cgs_id prvdrnum schedule admsndate dschrgdate cvrchrg prmdgnscd dgnscd2_25 ///
                prcdrcd dstntncd age_cnt sex disablehac using ../temp/`input', clear
            order cgs_id prvdrnum schedule admsndate dschrgdate cvrchrg prmdgnscd dgnscd2_25 ///
                prcdrcd dstntncd age_cnt sex disablehac
            keep if cgs_id <= `end' & cgs_id > `start'
            local N`i' = _N
            if `i' == `num_input_files' {
                assert `N1'+`N2'+`N3'+`N4'+`N5'+`N6'+`N7'+`N8'+`N9'+`N10'+ ///
                    `N11'+`N12'+`N13'+`N14'+`N15'+`N16'+`N17'+`N18'+`N19'+_N == `num_obs'
            }
            export delimited using ../output_local/`input'/cgs_input_part`i', replace
        }
    }
end

*Execute
main
