version 12
adopath + ../external/lib/stata/gslab_misc/ado 
adopath + ../external/lib/stata/ado 
preliminaries

global min_pre_year = 1998
global max_pre_year = 2002

global min_mid_year = 2003
global max_mid_year = 2007

global min_post_year = 2007
global max_post_year = 2014

global pre_restrictions year(dschrgdt) >= 1998 & year(dschrgdt) < 2003
global mid_restrictions (year(dschrgdt) >= 2003 & year(dschrgdt) <= 2006) | (year(dschrgdt) == 2007 & month(dschrgdt) < 10)
global post_restrictions (year(dschrgdt) > 2007 & year(dschrgdt) < 2014) | (year(dschrgdt) == 2007 & month(dschrgdt) >= 10) | (year(dschrgdt) == 2014 & month(dschrgdt) <= 7)

program main
    divide_hmolist
    foreach facility in ach ltch {
        foreach period in pre mid post {
            sample_restrictions, ///
                facility(`facility') period(`period') year_restrictions(${`period'_restrictions})        
            create_pat_month_panel, facility(`facility') period(`period') 
            eligibility_restrictions, facility(`facility') period(`period')
            create_medpar_ach_sample, facility(`facility')  period(`period')
        }
    }
end

program divide_hmolist
    use ../external_links/denom_post/hmolist${min_post_year}_${max_post_year}, clear
    keep if record_year <= 2010
    save ../temp/hmolist2007_2010, replace
    use ../external_links/denom_post/hmolist${min_post_year}_${max_post_year}, clear
    keep if record_year > 2010
    save ../temp/hmolist2011_2014, replace
end 

program sample_restrictions
    syntax, year_restrictions(string) facility(string) period(string)

    use ../external_links/medpar/medpar_append_`period', clear
    keep if `facility'
    replace dschrgdt = admsndt + util_day if missing(dschrgdt)
    keep if `year_restrictions'

    ooo Use Death Date from the Denom file
    merge m:1 bene_id using ///
        ../external_links/death_dates/deathlist${min_pre_year}_${max_post_year}, ///
        assert(1 2 3) keep(1 3) nogen
    replace death_dt=deathdt if death_dt==. 
    drop deathdt //drop death date from medpar, use death date from denom

    ooo Sample Restrictions on PPS (for ACH only - LTCH restrictions later in pipeline)
    tab pps_ind
    if ach == 1 {
        keep if pps_ind==2
        assert loscnt >= util_day    
    }
    
    ooo Drop Medicare does not cover the entire stay
    gen medicare_partialcover = loscnt != util_day
    tab medicare_partialcover
    keep if medicare_partialcover == 0 

    ooo Drop the stay is less than 1 day
    gen lessthan1day = dschrgdt <= admsndt
    tab lessthan1day
    keep if lessthan1day == 0

    drop loscnt
    assert util_day == dschrgdt - admsndt

    ooo Tab individuals who qualified due to old age DO NOT DROP
    tab ms_cd

    ooo Drop when death date comes before discharge date - we think they are data errors
    gen deathdt_since_admsn = death_dt - admsndt
    gen deathdt_since_dschrg = death_dt - dschrgdt
    gen deathdt_dschrg_error = deathdt_since_dschrg < 0
    tab deathdt_dschrg_error
    keep if deathdt_dschrg_error == 0

    save ../temp/medpar_loaded_`facility'_`period', replace
end

program create_pat_month_panel
    syntax, facility(string) period(string)
    * Note that we use medparid year as identifiers instead of medparid because
    * in the pre-period, medparid's were repeated in different years
        use ../temp/medpar_loaded_`facility'_`period', clear
        keep medparid year bene_id admsndt dschrgdt
        foreach var in admsn dschrg {
            gen `var'_mo = month(`var'dt)
            gen `var'_yr = year(`var'dt)
        }
        gen length = dschrg_mo - admsn_mo + 1
        replace length = length + 12*(dschrg_yr - admsn_yr)
        expand length
        bys medparid year: gen month = admsn_mo if _n == 1
        bys medparid year: gen record_year = admsn_yr if _n == 1
        replace month = month[_n-1] + 1 if missing(month) & medparid == medparid[_n-1] ///
             & year == year[_n-1]
        replace record_year = record_year[_n-1] if missing(record_year) & ///
            medparid == medparid[_n-1] & year == year[_n-1]
        replace record_year = record_year + 1 if month > 12 & month <= 24
        replace record_year = record_year + 2 if month > 24 & month <= 36
        assert month <= 36
        replace month = month - 12 if month > 12 & month <=24
        replace month = month - 24 if month > 24 & month <=36
        assert month >= 1 & month <= 12
        bys medparid year: assert record_year == year(dschrgdt) if _n == _N
        bys medparid year: assert month == month(dschrgdt) if _n == _N
        keep medparid bene_id month record_year
        save ../temp/pat_month_panel_`facility'_`period', replace
end

program eligibility_restrictions
    syntax, facility(string) period(string)
        if "`period'" == "post" {
            use ../temp/pat_month_panel_`facility'_`period', clear
            keep if record_year <= 2010
            merge m:1 bene_id month record_year using ../temp/hmolist2007_2010, assert(1 2 3) ///
                keep(3) nogen
            save ../temp/hmomerged2007_2010_`facility'_`period', replace
            use ../temp/pat_month_panel_`facility'_`period', clear
            keep if record_year > 2010
            merge m:1 bene_id month record_year using ../temp/hmolist2011_2014, assert(1 2 3) ///
                keep(3) nogen
            append using ../temp/hmomerged2007_2010_`facility'_`period'
            keep medparid record_year
            rename record_year year
            bys medparid: keep if _n == 1
            save ../temp/hmolist_`facility'_`period', replace 
        }
        else {
            use ../temp/pat_month_panel_`facility'_`period', clear
            merge m:1 bene_id month record_year using ///
                ../external_links/denom_`period'/hmolist${min_`period'_year}_${max_`period'_year}, ///
                assert(1 2 3) keep(3) nogen
            keep medparid record_year
            rename record_year year
            bys medparid year: keep if _n == 1
            save ../temp/hmolist_`facility'_`period', replace
        }

        use ../temp/pat_month_panel_`facility'_`period', clear
        merge m:1 bene_id month record_year using ///
            ../external_links/denom_`period'/mcaidlist${min_`period'_year}_${max_`period'_year}, ///
            assert(1 2 3) keep(3) nogen
        rename record_year year
        keep medparid year
        bys medparid year: keep if _n == 1
        save ../temp/mcaidlist_`facility'_`period', replace      
end

program create_medpar_ach_sample
    syntax, facility(string) period(string)
        use ../temp/medpar_loaded_`facility'_`period', clear
        merge 1:1 medparid year using ../temp/hmolist_`facility'_`period', ///
            assert(1 2 3) keep(1 3)

        ooo Restrict to Individuals not HMO
        gen hmo=_merge==3
        tab hmo
        keep if _merge==1 //keep those not found in the list of HMO bene-month-years
        drop _merge

        merge 1:1 medparid year using ../temp/mcaidlist_`facility'_`period', ///
            assert(1 2 3) keep(1 3)
        ooo Tab Share of Individuals on Medicaid DO NOT DROP
        gen mcaid=_merge==3
        tab mcaid
        drop _merge

        gen mort60day_admsn=deathdt_since_admsn<=60
        gen mort90day_admsn=deathdt_since_admsn<=90

        gen calendar_year=year(dschrgdt)

        save_data ../output_local/compiled_`facility'_stays_`period', replace key(medparid year) ///
            log("../output/data_file_manifest.log")
end

*Execute
main
