version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado 

global calvary_num = "332006"
preliminaries

** Note that we use destination code to define continuous spells, with < 1 day between discharge & admission.
* These spells are then used to create different definitions of "source" and/or "destination" 
* for the LTCH-I and LTCH-II projects.

global medpar_pre "../temp/medpar_pre"
global medpar_mid "../temp/medpar_mid"
global medpar_post "../external_links/medpar/medpar_append_post"

program main
    append_years
    define_spell
    define_post_ltch_spells
end

program append_years
    * Append years after the final year of the sample to make sure the spell is complete
    use if year==2003 using ../external_links/medpar/medpar_append_mid, clear
    append using ../external_links/medpar/medpar_append_pre
    save ../temp/medpar_pre, replace

    use if year==2008 using ../external_links/medpar/medpar_append_post, clear
    append using ../external_links/medpar/medpar_append_mid
    save ../temp/medpar_mid, replace
end

program define_spell
    foreach period in pre mid post {
        use ${medpar_`period'}, clear
        
        * Fix a discharge date for Still-A-Patients so we can order correctly
        replace dschrgdt = admsndt + util_day if dstntncd==30
        sort bene_id dschrgdt admsndt
        * Then remove it so we don't get confused
        replace dschrgdt = . if dstntncd==30

        * Define spells of continuous stay
        bys bene_id (admsndt): gen spell = 1 if _n==1
        * If missing(dschrgdt[_n-1]) then the following condition will be true, 
        * which is good because we want to treat it as a new stay
        replace spell = 1 if bene_id==bene_id[_n-1] & admsndt - dschrgdt[_n-1] > 1
        by bene_id (admsndt): gen num_spell = sum(spell)
        egen bene_spell = concat(bene_id num_spell), punct("_")
        drop spell

        save_data ../output_local/all_spells_`period', replace key(medparid year) ///
            log("../output/data_file_manifest.log") 
    }
end

program define_post_ltch_spells
    foreach period in pre mid post {
        use ../output_local/all_spells_`period', clear
        * Keep only spells that ever stayed in an ltch
        bys bene_spell: egen ever_ltch = max(ltch)
        keep if ever_ltch == 1
        * order so that non-ltchs come before ltchs if admsndt and dschrgdt are same
        gen not_ltch = ltch == 0
        sort bene_spell admsndt dschrgdt not_ltch
        * Refine to stays after first ltch stay
        by bene_spell: gen after_ltch = sum(ltch)
        keep if after_ltch > 0
        drop after_ltch
        * assert that first stays in sequence are LTCH
        bys bene_spell (admsndt dschrgdt not_ltch): assert ltch==1 if _n==1
        drop not_ltch
        save_data ../output_local/post_ltch_spell_`period', replace key(medparid year) ///
            log("../output/data_file_manifest.log") 
    }
end

*EXECUTE
main

