version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries
global keepvars=          "acute_mdcr_pmt acute_bene_pmt acute_prmry_pmt acute_perdiem_pmt acute_cov_days acute_stays oip_mdcr_pmt oip_bene_pmt oip_prmry_pmt oip_perdiem_pmt oip_cov_days oip_stays snf_mdcr_pmt snf_bene_pmt snf_prmry_pmt snf_cov_days snf_stays hos_mdcr_pmt hos_prmry_pmt hos_cov_days hos_stays hh_mdcr_pmt hh_prmry_pmt hh_visits readmissions ptd_events ptd_fill_cnt ptd_total_rx_cst ptd_mdcr_pmt ptd_bene_pmt"
global keepvars_post_post=" acute_md acute_be acute_pr acute_pe acute_co acute_st oip_mdcr oip_bene oip_prmr oip_perd oip_cov_ oip_stay snf_mdcr snf_bene snf_prmr snf_cov_ snf_stay hos_mdcr hos_prmr hos_cov_ hos_stay hh_mdcr_ hh_prmry hh_visit readmiss ptd_even ptd_fill ptd_tota ptd_mdcr ptd_bene"
global carrier_op_keepvars=          "hop_mdcr_pmt hop_bene_pmt hop_prmry_pmt  asc_mdcr_pmt asc_bene_pmt asc_prmry_pmt ptb_drug_mdcr_pmt ptb_drug_bene_pmt ptb_drug_prmry_pmt em_mdcr_pmt em_bene_pmt em_prmry_pmt anes_mdcr_pmt anes_bene_pmt anes_prmry_pmt dialys_mdcr_pmt dialys_bene_pmt dialys_prmry_pmt oproc_mdcr_pmt oproc_bene_pmt oproc_prmry_pmt img_mdcr_pmt img_bene_pmt img_prmry_pmt test_mdcr_pmt test_bene_pmt test_prmry_pmt dme_mdcr_pmt dme_bene_pmt dme_prmry_pmt othc_mdcr_pmt othc_bene_pmt othc_prmry_pmt phys_mdcr_pmt phys_bene_pmt phys_prmry_pmt"
global carrier_op_keepvars_post_post="hop_mdcr hop_bene hop_prmr asc_mdcr asc_bene asc_prmr ptbrxmp ptbrxbp ptbrxpp em_mdcr_ em_bene_ em_prmry anes_mdc anes_ben anes_prm dialys_m dialys_b dialys_p oproc_md oproc_be oproc_pr img_mdcr img_bene img_prmr test_mdc test_ben test_prm dme_mdcr dme_bene dme_prmr othc_mdc othc_ben othc_prm phys_mdc phys_ben phys_prm"

global min_year_pre=1999
global max_year_pre=2002

global min_year_mid = 2003
global max_year_mid = 2007

global min_year_post=2007
global max_year_post=2014

program main
    load_CU
end

program load_CU
    foreach period in pre mid post {
        forval year=${min_year_`period'}/${max_year_`period'}  {
            if `year' == 2013 | `year' == 2014 {
                use bene_id rfrnc_yr ${keepvars_post_post} ${carrier_op_keepvars_post_post} using ../external/cu/bsfcu`year', clear
                rename rfrnc_yr bene_enrollmt_ref_yr
                rename (${keepvars_post_post} ${carrier_op_keepvars_post_post}) (${keepvars} ${carrier_op_keepvars})
            }
            else {
                use bene_id bene_enrollmt_ref_yr ${keepvars} ${carrier_op_keepvars} using ../external/cu/bsfcu`year', clear
            }
            drop if missing(bene_id) | missing(bene_enrollmt_ref_yr)
            gen calendar_year=bene_enrollmt_ref_yr
            *keep in mind that events are defined for CY not RY so this is not very precise
            duplicates drop
            * drop observations whose variables are all missing to save space
            gen drp = 1
            foreach var in ${keepvars} ${carrier_op_keepvars} {
                replace drp = 0 if ~missing(`var')
            }
            drop if drp==1
            drop drp
            save ../temp/CU`year'_loaded, replace
            keep bene_id calendar_year ${keepvars}
            save ../temp/CU`year', replace
	        use ../temp/CU`year'_loaded, replace
            keep bene_id calendar_year ${carrier_op_keepvars}
            save ../temp/CU`year'_car_op, replace
        }
    }
    clear
    foreach period in pre mid post {
        forval year=${min_year_`period'}/${max_year_`period'} {
            append using ../temp/CU`year'
        }
        drop if missing(calendar_year)
        save_data ../output_local/CU_append_`period', replace key(bene_id calendar_year) ///
            log("../output/data_file_manifest.log")
        clear
        forval year=${min_year_`period'}/${max_year_`period'} {
            append using ../temp/CU`year'_car_op
        }
        drop if missing(calendar_year)
        save_data ../output_local/CU_append_car_op_`period', replace key(bene_id calendar_year) ///
            log("../output/data_file_manifest.log") 
        clear
    }

end

*Execute
main
