version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado 

preliminaries

program main
    label_src_claims
    add_medpar_defns
    combine_defns
end

program label_src_claims
    foreach period in pre post {
        use medparid year admsndt dschrgdt bene_spell ach ltch snf irf src_adms ///
            using ../external_links/spells/all_spells_`period', clear

        sort bene_spell dschrgdt admsndt
        foreach hosp in ach ltch snf irf {
            gen source_`hosp' = 1 if `hosp'[_n-1]==1 & bene_spell==bene_spell[_n-1] 
            gen dest_`hosp' = 1 if `hosp'[_n+1]==1 & bene_spell==bene_spell[_n+1] 
        }

        gen src_init = 1 if source_ach==1
        replace src_init = 2 if source_ltch==1
        replace src_init = 3 if source_snf==1
        replace src_init = 4 if source_irf==1
        replace src_init = 5 if missing(src_init)
        label define src_labels 1 "ACH" 2 "LTCH" 3 "SNF" 4 "IRF" 5 "Other"
        label values src_init src_labels
        label variable src_init "Admission source according to preceding MedPAR stay"

        drop source_ach source_ltch source_snf source_irf ///
                dest_ach dest_ltch dest_snf dest_irf

        ooo Constructed Spell Definition
        tab src_init
        save ../temp/claims_defined_origins_`period', replace
    }
end

program add_medpar_defns
    foreach period in pre post {
        use ../temp/claims_defined_origins_`period', clear
        * MedPAR defined source/dest definitions
        gen OrigType=1 if inlist(src_adms, "1","2","3") //referral
        replace OrigType=2 if inlist(src_adms, "4","A") //transfer from another hospital
        replace OrigType=3 if src_adms=="5" //transfer from SNF ICF (intermediate care facility)
        replace OrigType=4 if src_adms=="D" //transfer from same hospital
        replace OrigType=5 if src_adms=="7" //transfer from ER
        replace OrigType=6 if inlist(src_adms, "6", "8", "B" ,"C", "E", "F") //other facility,law enf
        replace OrigType=7 if inlist(src_adms, "0", "9", ",", "R", "X", "W") //unknown
        replace OrigType=7 if missing(src_adms)
        cap label define orig_types 1 "Referral" 2 "Transfer from Another Hospital" ///
            3 "Transfer from SNF/ICF" 4 "Transfer from same Hospital" ///
            5 "Transfer from ER" 6 "Other" 7 "Unknown"
        label values OrigType orig_types
        label variable OrigType "Admission source according to MedPAR admission variable"

        ooo MedPAR vs Constructed Spell Definition - Source
        tab OrigType src_init, matcell(medpar_vs_init_`period')

        ooo admissions_medpar definition
        tab OrigType, matcell(medpar_`period')

        ooo admissions_constructed_initial definition
        tab src_init, matcell(init_`period')

        matrix_to_txt, saving(../output/tables/medpar_vs_init_`period'.txt) ///
            matrix(medpar_vs_init_`period') format(%20.4f) replace ///
            title(<tab:medpar_vs_init_`period'>)

        matrix_to_txt, saving(../output/tables/medpar_`period'.txt) ///
            matrix(medpar_`period') format(%20.4f) replace ///
            title(<tab:medpar_`period'>)

        matrix_to_txt, saving(../output/tables/init_`period'.txt) ///
            matrix(init_`period') format(%20.4f) replace ///
            title(<tab:init_`period'>)

        ooo LTCH MedPAR vs Constructed Spell Definition - Source
        tab OrigType src_init if ltch==1

        save ../temp/claims_medpar_defined_origins_`period', replace
    }
end

program combine_defns
    foreach period in pre post {
        use ../temp/claims_medpar_defined_origins_`period', clear
        * Update source definition by combining MedPAR and constructed-spell definition
        gen source_type = 1 if OrigType==1 // referral
        replace source_type=2 if OrigType==5 // ER
        replace source_type=3 if src_init==1 & missing(source_type) // ACH
        replace source_type=4 if src_init==2 & missing(source_type) // LTCH
        replace source_type=5 if src_init==3 & missing(source_type) // SNF
        replace source_type=6 if src_init==4 & missing(source_type) // IRF
        replace source_type=7 if OrigType==2 & missing(source_type) // another hospital
        replace source_type=8 if missing(source_type) // Other
        label define src_lab 1 "Referral" 2 "Transfer from ER" 3 "Transfer from ACH" ///
            4 "Transfer from LTCH" 5 "Transfer from SNF" 6 "Transfer from IRF" ///
            7 "Transfer from Another Hospital" 8 "Other"
        label values source_type src_lab
        label variable source_type ///
            "Admission source based on combining MedPAR variable and previous spell definitions"

        ooo New Source Definition - Combined MedPAR and Constructed Spell
        tab source_type
        ooo MedPAR Origin of Other observations
        tab OrigType if source_type==8

        gen OrigReferral=source_type==1
        gen OrigER=source_type==2
        gen OrigACH=source_type==3
        gen OrigLTCH=source_type==4
        gen OrigSNF=source_type==5
        gen OrigIRF=source_type==6
        gen OrigOtherHosp=source_type==7
        gen OrigOther=source_type==8
        assert OrigReferral+OrigER+OrigACH+OrigLTCH+OrigSNF+OrigIRF+OrigOtherHosp+OrigOther==1

	drop OrigType
        keep medparid year Orig*
        save_data ../output_local/orig_from_spells_`period', replace key(medparid year) ///
            log("../output/data_file_manifest.log")   
    }

end

*EXECUTE
main

