version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
preliminaries

global min_sample_year_post=2007
global max_sample_year_post=2014


program main
    append_denom
    generate_ziplist
    generate_hmo_list
    generate_mcaid_list
end

program append_denom
    forval year=${min_sample_year_post}/${max_sample_year_post} {
        use ../external/denom/bsfab`year', clear
        keep bene_id bene_zip death_dt hmoind* hmo_mo buyin* 
        gen record_year=`year'
        save ../temp/denom`year', replace
    }
    clear
    forval year=${min_sample_year_post}/${max_sample_year_post} {
        append using ../temp/denom`year'
    }
    save ../temp/denom_post, replace
end

program generate_ziplist
    use bene_id record_year bene_zip using ../temp/denom_post, clear
    drop if missing(bene_zip)
    save_data ../output_local/ziplist${min_sample_year_post}_${max_sample_year_post}, ///
        replace key(bene_id record_year) log("../output/data_file_manifest.log")  
end

program generate_hmo_list
    use bene_id hmoind* hmo_mo record_year using ../temp/denom_post, clear
    drop if hmo_mo==0
    drop hmo_mo
    renvars hmoind0*,pres(hmoind0 hmoind)
    expand 12
    bys bene_id record_year: gen n = _n
    gen hmoind = . 
    tostring hmoind, replace
    gen month = .
    forval i = 1/12 {
        replace hmoind = hmoind`i' if n == `i'
        replace month = `i' if n == `i'
        drop hmoind`i'
    }
    drop n
    gen hmo=hmoind!="0"
    keep if hmo==1
    save_data ../output_local/hmolist${min_sample_year_post}_${max_sample_year_post}, ///
        replace key(bene_id record_year month) ///
        log("../output/data_file_manifest.log")
end

program generate_mcaid_list
    use bene_id buyin* record_year using ../temp/denom_post, clear
    drop if buyin_mo==0
    drop buyin_mo
    renvars buyin0**,pres(buyin0 buyin)
    expand 12
    bys bene_id record_year: gen n = _n
    gen buyin = . 
    tostring buyin, replace
    gen month = .
    forval i = 1/12 {
        replace buyin = buyin`i' if n == `i'
        replace month = `i' if n == `i'
        drop buyin`i'
    }
    drop n
    gen mcaid=inlist(buyin,"A","B","C")
    keep if mcaid==1
    save_data ../output_local/mcaidlist${min_sample_year_post}_${max_sample_year_post}, ///
        replace key(bene_id record_year month) ///
        log("../output/data_file_manifest.log")
end


*Execute
main
