version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
preliminaries

global min_sample_year_pre=1998
global max_sample_year_pre=2002

global min_sample_year_mid=2003
global max_sample_year_mid=2007

global min_sample_year_post=2007
global max_sample_year_post=2014

global calvary_num = "332006"

global all_harmonized_vars="acmdtns acrtndt ad_dgns admsnday admsndt age_cnt amblnc ansthsa bene_id bene_zip bic bldadmin blddedam bldfrnsh bloodamt clnc_amt cnty_cd coin_amt coin_day crdlgy crnry_cd crnryamt crnryday ctscansw cvrchrg cvrlvldt deathcd deathday deathdt ded_amt dgns_poa dgnscd1 dgnscd10 dgnscd2 dgnscd3 dgnscd4 dgnscd5 dgnscd6 dgnscd7 dgnscd8 dgnscd9 dgnscnt dgnstcsw disp_shr dme_amt dprtmntl drg_cd drgprice dschrgcd dschrgdt dstntncd eq_bic er_amt esrd_cd esrdsetg esrdstg1 esrdstg2 esrdstg3 esrdstg4 esrdstg5 exhst_dt faclmcnt ficarr fildtcd ghopdcd icareamt icarecnt icuindcd ime_amt imgng_sw inhltamt ipsbcd lab_amt loscnt lrd_use lthtrpsy medparid mri_amt ms_cd nuclr_sw ocptlamt onclgysw opsrvc opsrvccd orgnamt orgncd orgnpinm oroomamt othramt outlr_cd outlramt outlrday passthru phrmcamt phrmcycd phythamt pmt_amt pps_cptl pps_ind prcdrcd1 prcdrcd2 prcdrcd3 prcdrcd4 prcdrcd5 prcdrcd6 prcdrcnt prcdrdt1 prcdrdt2 prcdrdt3 prcdrdt4 prcdrdt5 prcdrdt6 prcdrsw prcdtcnt proffees prpay_cd prpayamt prvdrnum prvtamt prvtday qlfyfrom qlfythru race rdlgyamt sex smplsize spch_amt spclunit sprvtamt sprvtday src_adms ssicd ssiday sslssnf state_cd suplyamt thrptcsw totchrg trnsplnt type_adm udme_amt util_day wardamt wardday wrngcd"
global keep_vars="medparid bene_id prvdrnum ms_cd state_cd cnty_cd bene_zip pps_ind age_cnt race sex drg_cd src_adms dstntncd admsndt dschrgdt loscnt deathdt deathcd pmt_amt passthru blddedam coin_amt ded_amt prpayamt outlr_cd outlramt util_day gho cvrchrg pps_cptl disp_shr ime_amt drgprice dschrgcd src_adms"

program main
    append_medpar
end

program append_medpar
    foreach period in pre mid post {
        forval year=${min_sample_year_`period'}/${max_sample_year_`period'} {
            use ${keep_vars} using ../external/medpar/med`year', clear
            gen year=`year'
            label variable year "MedPAR record year"
            gen provider_id = substr(prvdrnum,3,.)
            destring provider_id, replace force
            gen ach  = inrange(provider_id, 1, 999) | ///
                        inrange(provider_id, 1300, 1399) 
            gen ltch = inrange(provider_id, 2000, 2299)
            gen snf  = inrange(provider_id, 5000, 6499)
            gen irf  = inrange(provider_id, 3025, 3099)
            gen hsp  = inrange(provider_id, 1500, 1799)
            gen hha  = inrange(provider_id, 7000, 7299) | ///
                       inrange(provider_id, 7400, 7799) | ///
                       inrange(provider_id, 8000, 8499) | ///
                       inrange(provider_id, 9000, 9799)
            assert hha==0
            assert hsp==0
            gen prov_type=1 if ach==1
            replace prov_type=2 if ltch==1
            replace prov_type=3 if snf==1
            replace prov_type=4 if irf==1
            replace prov_type=5 if missing(prov_type)
            label define type_labels 1 "ACH" 2 "LTCH" 3 "SNF" 4 "IRF" 5 "Other (non-HHA/hospice)"
            label values prov_type type_labels
            foreach var in ms_cd pps_in drg_cd dstntncd {
                destring `var', replace
            }

            gen DestType=1 if inlist(dstntncd,1,6,8)
            replace DestType=2 if inlist(dstntncd,2,5,9,43,66)
            replace DestType=3 if inlist(dstntncd,3,62)
            replace DestType=4 if inlist(dstntncd,50,51)
            replace DestType=5 if inlist(dstntncd,20,40,41,42)
            replace DestType=6 if inlist(dstntncd,63)
            replace DestType=7 if inlist(dstntncd,4,61,64,65,70,71,72)
            replace DestType=8 if inlist(dstntncd,30)
            replace DestType=7 if missing(DestType)
            cap label define Dest_labels 1 "Home/home health(excl. hospice)" ///
                2 "Inpatient Care" 3 "SNF/IRF" 4 "Hospice (home or facility)" ///
                5 "Expired" 6 "LTCH" 7 "Other" 8 "Still A Patient"
            label values DestType Dest_labels
            label variable DestType "Destination according to MedPAR destination variable"

            gen DestHomeCare=DestType==1
            gen DestInpat=DestType==2
            gen DestSNFIRF=DestType==3
            gen DestHospice=DestType==4
            gen DestDeath=DestType==5
            gen DestLTCH=DestType==6
            gen DestOther=DestType==7
            gen StillPatient=DestType==8
            assert DestHomeCare + DestInpat + DestSNFIRF + DestHospice + DestLTCH + ///
                DestDeath + DestOther + StillPatient==1
            label variable DestHomeCare "Destination code indicates home/home health"
            label variable DestInpat "Destination code indicates inpatient"
            label variable DestSNFIRF "Destination code indicates SNF/IRF"
            label variable DestHospice "Destination code indicates hospice"
            label variable DestDeath "Destination code indicates death"
            label variable DestLTCH "Destination code indicates LTCH (introduced 2001)"
            label variable DestOther "Destination code indicates other"
            label variable StillPatient "Destination code indicates still a patient (SNF/IRF only)"

            save ../temp/medpar`year', replace
        }
        clear
        forval year=${min_sample_year_`period'}/${max_sample_year_`period'} {
            append using ../temp/medpar`year'
        }

        * Drop Calvary hospital (which is omitted from the analysis)
        drop if prvdrnum=="${calvary_num}"

        di "`period' period:"
        tab prov_type

        save_data ../output_local/medpar_append_`period', replace key(medparid year) ///
            log("../output/data_file_manifest.log")
    }
end

** EXECUTE
main
