version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
preliminaries

global pre_data = "../external_links/all_spells/all_spells_pre"
global mid_data = "../external_links/all_spells/all_spells_mid"
global post_data = "../external_links/all_spells/all_spells_post"

program main
    home_90days
    split_by_type
end

program home_90days
    foreach period in pre mid post {
        use medparid year bene_id admsndt dschrgdt deathdt util_day ach ltch snf irf ///
            bene_spell DestHomeCare ///
            using ${`period'_data}, clear

        gen DestNotHome = 1 - DestHomeCare
        sort bene_spell DestNotHome
        by bene_spell: gen spell_dschrg_date = dschrgdt[1] if DestHomeCare[1]==1
        gen days_before_home = spell_dschrg_date - admsndt
        gen home_90days = days_before_home <= 90
        replace home_90days = . if days_before_home < 0
        
        keep medparid year bene_id bene_spell home_90days ach ltch snf irf
        save ../temp/home_90days_`period', replace
    }
end

program split_by_type
    foreach period in pre mid post {
        use medparid year bene_id bene_spell home_90days ltch ///
            if ltch==1  using ../temp/home_90days_`period', clear
        drop ltch
        save_data ../output_local/home_90days_`period'_LTCH, replace key(medparid year) ///
            log("../output/data_file_manifest.log")

        use medparid year bene_id bene_spell home_90days ach ///
            if ach==1 using ../temp/home_90days_`period', clear
        drop ach
        save_data ../output_local/home_90days_`period'_ACH, replace key(medparid year) ///
            log("../output/data_file_manifest.log")

        use medparid year bene_id bene_spell home_90days snf irf ///
           if snf==1|irf==1 using ../temp/home_90days_`period', clear
        drop snf irf
        save_data ../output_local/home_90days_`period'_SNF_IRF, replace key(medparid year) ///
            log("../output/data_file_manifest.log")
    }
end

* EXECUTE
main
