version 12
adopath + ../external/lib/stata/gslab_misc/ado 
adopath + ../external/lib/stata/ado 
preliminaries

global min_sample_year_pre = 1998
global max_sample_year_pre = 2002

global min_sample_year_mid = 2003
global max_sample_year_mid = 2007

global min_sample_year_post = 2007
global max_sample_year_post = 2014

global all_harmonized_vars="acmdtns acrtndt ad_dgns admsnday admsndt age_cnt amblnc ansthsa bene_id bene_zip bic bldadmin blddedam bldfrnsh bloodamt clnc_amt cnty_cd coin_amt coin_day crdlgy crnry_cd crnryamt crnryday ctscansw cvrchrg cvrlvldt deathcd deathday deathdt ded_amt dgns_poa dgnscd1 dgnscd10 dgnscd2 dgnscd3 dgnscd4 dgnscd5 dgnscd6 dgnscd7 dgnscd8 dgnscd9 dgnscnt dgnstcsw disp_shr dme_amt dprtmntl drg_cd drgprice dschrgcd dschrgdt dstntncd eq_bic er_amt esrd_cd esrdsetg esrdstg1 esrdstg2 esrdstg3 esrdstg4 esrdstg5 exhst_dt faclmcnt ficarr fildtcd ghopdcd icareamt icarecnt icuindcd ime_amt imgng_sw inhltamt ipsbcd lab_amt loscnt lrd_use lthtrpsy medparid mri_amt ms_cd nuclr_sw ocptlamt onclgysw opsrvc opsrvccd orgnamt orgncd orgnpinm oroomamt othramt outlr_cd outlramt outlrday passthru phrmcamt phrmcycd phythamt pmt_amt pps_cptl pps_ind prcdrcd1 prcdrcd2 prcdrcd3 prcdrcd4 prcdrcd5 prcdrcd6 prcdrcnt prcdrdt1 prcdrdt2 prcdrdt3 prcdrdt4 prcdrdt5 prcdrdt6 prcdrsw prcdtcnt proffees prpay_cd prpayamt prvdrnum prvtamt prvtday qlfyfrom qlfythru race rdlgyamt sex smplsize spch_amt spclunit sprvtamt sprvtday src_adms ssicd ssiday sslssnf state_cd suplyamt thrptcsw totchrg trnsplnt type_adm udme_amt util_day wardamt wardday wrngcd"
global keep_vars="medparid bene_id prvdrnum ms_cd state_cd cnty_cd bene_zip pps_ind age_cnt race sex drg_cd src_adms dstntncd admsndt dschrgdt loscnt deathdt deathcd pmt_amt passthru blddedam coin_amt ded_amt prpayamt outlr_cd outlramt util_day gho cvrchrg pps_cptl disp_shr ime_amt drgprice dschrgcd src_adms"

program main
    topcode_proc_diag
    merge_procedures
    save_dta
end

program topcode_proc_diag
     * in 2011 CMS started allowing up to 25 procedure and diagnosis codes (relative to 6/9 procedures/diagnoses before)
     * we drop all procedures in the sequence after 6 procedures and 9 diagnoses for consistency 
    foreach period in pre mid post {
        forval year=${min_sample_year_`period'}/${max_sample_year_`period'} {
            use ../external/medpar/prcdr`year', clear
            gen year = `year'
            keep bene_id medparid prcdrseq prcdrcd prcdrdt year
            drop if prcdrseq > 6
            save ../temp/proc`year', replace
            
            use ../external/medpar/dgnscd`year', clear
            gen year = `year' 
            drop if dgnscdseq > 9 
            save ../temp/diag`year', replace
        }

        clear
        forval year=${min_sample_year_`period'} / ${max_sample_year_`period'} {
            append using ../temp/proc`year'
        }
        save_data ../output_local/proc_`period', replace key(medparid year prcdrseq) ///
            log("../output/data_file_manifest.log")
	
        clear
        forval year=${min_sample_year_`period'} / ${max_sample_year_`period'} {
            append using ../temp/diag`year'
        }

        save_data ../output_local/diag_`period', replace key(medparid year dgnscdseq) ///
            log("../output/data_file_manifest.log")
    }
end

program merge_procedures
    foreach period in pre mid post {
        foreach dta in proc diag {
            use ../output_local/`dta'_`period', clear
            merge m:1 medparid year using ../external_links/medpar/medpar_append_`period', ///
                keepusing(medparid year ach ltch snf irf prov_type) assert(1 2 3) keep(3) nogen
            save ../temp/`dta'_append_`period', replace
        }
    }
end

program save_dta
    foreach period in pre mid post {
        foreach dta in proc diag {
            foreach dataset in ltch ach snf irf {
                use ../temp/`dta'_append_`period'
                keep if `dataset'==1
                save ../temp/`dta'_append_`period'_`dataset', replace
            }
            use ../temp/`dta'_append_`period'_snf, clear
            append using ../temp/`dta'_append_`period'_irf
            save_data ../output_local/`dta'_append_`period'_snf_irf, ///
                key(medparid year *seq) replace ///
                log("../output/data_file_manifest.log")

            use ../temp/`dta'_append_`period'_ach, clear
            save_data ../output_local/`dta'_append_`period'_ach, ///
                key(medparid year *seq) replace ///
                log("../output/data_file_manifest.log")

            use ../temp/`dta'_append_`period'_ltch, clear
            save_data ../output_local/`dta'_append_`period'_ltch, ///
                key(medparid year *seq) replace ///
                log("../output/data_file_manifest.log")
        }
    }
end

*EXECUTE
main
