version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado

preliminaries

program main
    obtain_CCR
    combine_output
end

program obtain_CCR
    use prvdrnum beginschedule costtochargeratio ///
        using ../external/parameter/cgs_ltch_param, clear
    encode prvdrnum, gen(provider)
    tsset provider beginschedule
    tsfill
    sort provider beginschedule
    carryforward costtochargeratio, replace
    carryforward prvdrnum, replace
    foreach date in month day {
        gen schedule_`date'=`date'(beginschedule)
        tostring schedule_`date', replace format(%02.0f)
    }
    gen schedule_year=year(beginschedule)
    tostring schedule_year, replace format(%04.0f)
    egen cgs_dischargedate=concat(schedule_month schedule_day schedule_year)
    destring cgs_dischargedate, replace
    drop provider
    save ../temp/ccr, replace
end

program combine_output
    forv i=1/20 {
        insheet using ../../file`i'/output/cgs_output_part`i'.csv, clear
        save ../temp/cgs_output_part`i', replace
    }
    clear
    forv i=1/20 {
        append using ../temp/cgs_output_part`i'
    }
    foreach var in * {
        rename `var' cgs_`var'
    }
    rename cgs_accountnumber cgs_id
    drop cgs_userkey1
    merge 1:1 cgs_id using ../external_links/input/cgs_id_xwalk, assert(3) keep(3) nogen
    merge n:1 id using ../external_links/input/original_data, assert(3) keep(3) nogen
    merge n:1 prvdrnum cgs_dischargedate using ../temp/ccr, assert(1 2 3) keep(1 3)

    ooo Drop Obs not Matched to CCR
    gen ccr_nonmatch=_merge==1
    drop _merge
    egen ccr_nonmatch_ids=max(ccr_nonmatch)
    unique id if ccr_nonmatch==1
    drop if ccr_nonmatch==1
	
    ooo Accuracy of DRG Assignment
    gen drg_correct=cgs_drg==LTCDRG_Code
    tab drg_correct if fakedata==0
    
    ooo Accuracy of Payment Calculation
    gen payment_correct=abs(cgs_totalpayment-drgprice-outlramt)<=1
    tab payment_correct if fakedata==0

    ooo Keep DRG correct and payment correct
    foreach var in drg_correct payment_correct {
        bys id: egen `var'_keep=max(`var')
    }
    keep if drg_correct_keep==1 & payment_correct_keep==1
	
    save_data ../output_local/cgs_output, replace key(cgs_id) ///
        log("../output/data_file_manifest.log")
end

*Execute
main
