% Choose the version of params_all vector here and define other sub-params
% vectors. We need params_all vector to have the correct dimension because
% it affects how the simplex is set up.

%this function takes a vector 'params_all', and assigm them to each
%model parameter according to the model version

%whichrun: pool, pre or post
function [params_health_pre, params_health_post, params_pref, param_scale, ...
          zeta_a_pre, zeta_s_pre, zeta_a_post, zeta_s_post] ...
          = assign_params(params_all, paramsStruct)

    params_all = params_all(:)'; % Force this to be a row vector.
    estType    = paramsStruct.estType;
    paramsEst  = paramsStruct.paramsEstimated;
    momentsUsed = paramsStruct.momentsUsed;
    %%
    %model:
    %a3: basecase
    %a5: time-vayring rho
    
    if strcmp(estType, 'a5')  
        health_pars = {'mu', 'rho', 'rho1', 'sigma', 'mu0', 'sigma0'};
    else
        health_pars = {'mu', 'rho', 'sigma', 'mu0', 'sigma0'};    
    end
    
    pref_pars = {'gamma', 'v1A', 'v0S', 'v1S'};
    

    %% populate LTCH parameter lists
    nhealthpar = length(health_pars);
    nprefpar   = length(pref_pars);
    

    %health parameter
    params_health_pre =zeros(1,nhealthpar);
    params_health_post=zeros(1,nhealthpar);
    for i=1:nhealthpar
        if ~strcmp(momentsUsed, 'post')
            params_health_pre(i)  = params_all(strcmp(paramsEst, [health_pars{i}, 'Pre']));
        end
        if ~strcmp(momentsUsed, 'pre')
            params_health_post(i) = params_all(strcmp(paramsEst, [health_pars{i}, 'Post']));       
        end
    end
    
    %preference parameter
    params_pref=zeros(1,nprefpar);
    for i=1:nprefpar
        params_pref(i)= 1e3*params_all(strcmp(paramsEst, pref_pars{i}));       
    end    
    
    %logit error term
    param_scale = 1e2*params_all(strcmp(paramsEst, 'scale'));
    
    %% populate post LTCH parameter lists
    zeta_a_pre  = [params_all(strcmp(paramsEst, 'zeta0APre')),  params_all(strcmp(paramsEst, 'zeta1APre'))];  
    zeta_s_pre  = [params_all(strcmp(paramsEst, 'zeta0SPre')),  params_all(strcmp(paramsEst, 'zeta1SPre'))];  
    zeta_a_post = [params_all(strcmp(paramsEst, 'zeta0APost')), params_all(strcmp(paramsEst, 'zeta1APost'))];  
    zeta_s_post = [params_all(strcmp(paramsEst, 'zeta0SPost')), params_all(strcmp(paramsEst, 'zeta1SPost'))];  

end