function cfRetStruct = calculate_counterfactual_payments_allCF(...
    dischargePat, cfParamsStruct, cfRetStruct, scenariosAll, cf_scenarios)

%------- Calculate counterfactual payments under different payment schedules.
nonltchPayModel = cfParamsStruct.nonltchPayModel;

    iter = 1;
    for scenarioID = {scenariosAll{:}}
        scenarioID = cell2mat(scenarioID);
        [temp_output, cfRetStruct] = calculate_counterfactual_payments(...
            scenarioID, dischargePat, cfParamsStruct, cfRetStruct);

        T_cf = cfParamsStruct.T_cf;
        p_cf = cfRetStruct.(['p_' scenarioID]);
        p_postSSO = cfRetStruct.(['p_postSSO_' scenarioID]);

        revenueLGRI = cfRetStruct.payments_l_post_baseline; 
        capDay = cf_scenarios(scenarioID,'T').T;
        capDay_rel_SSO = capDay - T_cf;
        perdiem_preKink = p_cf(2) - p_cf(1);
        capAmount = p_cf(capDay);
        if p_postSSO >0
            capAmount = NaN;
        end

        temp_output = [temp_output; revenueLGRI; capDay; capDay_rel_SSO;...
            perdiem_preKink; capAmount];

        eval(['output_' scenarioID '= temp_output;']);

        if iter ==1
            outputCols = ['output_' scenarioID];
        else
            outputCols = [outputCols ', ' ['output_' scenarioID]];
        end
        iter = iter + 1;
    end

    %------- Assemble table to output.
    % Row names for counterfactual systemwide payments table.
    temp_cf_systemPay_rowNames = {'share_l_not_disc_60days'; 'share_l_never_disc'; 'tot_payments'; ...
        'payments_l'; 'profits_l'; 'alos'; 'payments_l_per_day'; 'profits_l_per_day'; ...
        'tot_payments_a'; 'tot_share_a'; 'avg_payments_a'; 'avg_days_a'; 'avg_health_a'; ...
        'tot_payments_s'; 'tot_share_s'; 'avg_payments_s'; 'avg_days_s'; 'avg_health_s'; ...
        'tot_share_m'; ...
        'tot_share_a_preSSO'; 'tot_share_a_postSSO'; 'avg_health_a_preSSO'; 'avg_health_a_postSSO'; ...
        'tot_payments_a_preSSO'; 'tot_payments_a_postSSO'; ...
        'avg_payments_a_preSSO'; 'avg_payments_a_postSSO'; ...
        'avg_days_a_preSSO'; 'avg_days_a_postSSO'; ...
        'tot_share_s_preSSO'; 'tot_share_s_postSSO'; 'avg_health_s_preSSO'; 'avg_health_s_postSSO'; ...
        'tot_payments_s_preSSO'; 'tot_payments_s_postSSO'; ...
        'avg_payments_s_preSSO'; 'avg_payments_s_postSSO'; ...
        'avg_days_s_preSSO'; 'avg_days_s_postSSO'; ...
        'tot_share_m_preSSO'; 'tot_share_m_postSSO'; ...
        'revenueLGRI'; 'capDay'; 'capDay_rel_SSO'; 'perdiem_preKink'; 'capAmount'};

    % Create a table of counterfactual systemwide payments across all scenarios.
    temp_cf_systemPay = eval(['table(' outputCols ...
        ', ''RowNames'', temp_cf_systemPay_rowNames)']);
    cfRetStruct.(['cf_systemPay_' nonltchPayModel dischargePat])= temp_cf_systemPay;


    % Write table of counterfactual systemwide payments to Excel.
    xlsSheetName = ['sysPay' cfParamsStruct.xlsShtNm '_' nonltchPayModel dischargePat];
    xlsOutFile = cfParamsStruct.xlsOutFile;
    writetable(temp_cf_systemPay, xlsOutFile, 'Sheet', xlsSheetName,'WriteRowNames',true);

    %print time stamp
    print_timestamp(xlsOutFile, xlsSheetName, cfParamsStruct.version, cfParamsStruct.run);

end