function calculate_post_mort(T, cfParamsStruct, cfRetStruct, scenariosAll)

    %------- Calculate T days mortality post discharge
    days = (1:cfParamsStruct.TT)' - cfParamsStruct.backInductT;
    nT=length(days);

    mortOut=days;
    varName={'days'};
    for scenarioID = {scenariosAll{:}}
        scenarioID = cell2mat(scenarioID);

        %health distribution at dishcharge each day
        h_a_dist = cfRetStruct.(['h_a_dist_post_', scenarioID]);
        h_s_dist = cfRetStruct.(['h_s_dist_post_', scenarioID]);
        PD_Struct= cfRetStruct.(['P_D_post_', scenarioID]);
        if isstruct(PD_Struct)
            PD_name = fieldnames(PD_Struct);
            PD = PD_Struct.(PD_name{1});
        else
            PD = PD_Struct;
        end
        %T day mortality
        ndays = size(h_s_dist,1);
        mortT_a= [zeros(ndays,1), h_a_dist]*(PD^(T-1))*PD(:,1);
        mortT_s= [zeros(ndays,1), h_s_dist]*(PD^(T-1))*PD(:,1);

        %concat output
        mortOut=[mortOut, mortT_a(1:nT,1), mortT_s(1:nT,1)];
        varName=[varName, {[scenarioID, '_up']}, {[scenarioID, '_down']}];
    end

    %------- Assemble table to output.
    if strcmpi(cfParamsStruct.version, 'v18_a3_pool') || strcmpi(cfParamsStruct.version, 'v17_a9_pool')
        % Write table of counterfactual systemwide payments to Excel.
        xlsSheetName = ['post_mort', cfParamsStruct.xlsShtNm];
        xlsOutFile = cfParamsStruct.xlsOutFile;

        %header
        xlswrite(xlsOutFile,varName,xlsSheetName,'A1');
        xlswrite(xlsOutFile,mortOut,xlsSheetName,'A2');

        %print time stamp
        print_timestamp(xlsOutFile, xlsSheetName, cfParamsStruct.version, cfParamsStruct.run);
    end
end
