function [moments_hat, ...
            share_a_pre_hat, share_s_pre_hat, share_l_pre_hat, share_m_pre_hat, ...
            z_pre, P_D_pre, h_0_pre, ...
            W_postSSO_prime_pre, q_postSSO_pre, V_A_pre, V_S_pre, ...
            Q_preSSO_pre, ...
            h_a_pre, h_s_pre, h_a_dist_pre, h_s_dist_pre, ...
            H_pre, H_l_pre, Q_a_pre, Q_s_pre, Q_l_pre, ...
            share_a_post_hat, share_s_post_hat, share_l_post_hat, share_m_post_hat, ...
            z_post, P_D_post, h_0_post, ...
            W_postSSO_prime_post, q_postSSO_post, V_A_post, V_S_post, V_l_post, ...
            W_preSSO_post, Q_preSSO_post, l_preSSO_post, ...
            h_a_post, h_s_post, h_a_dist_post, h_s_dist_post, ...
            H_post, H_l_post, Q_a_post, Q_s_post, Q_l_post, ...
            nonltchPay_a_pre_hat,  nonltchPay_s_pre_hat, ...
            nonltchPay_a_post_hat, nonltchPay_s_post_hat ...            
            ] = collect_obj_results(params_all, paramsStruct)

    % Post-LTCH model
    nonltchPayModel = paramsStruct.nonltchPayModel;
    
    % Whether we use pre-PPS, post-PPS, or both sets of moments.
    momentsUsed = paramsStruct.momentsUsed;

    %% dynamic or static LOS model
    if ~strcmp(paramsStruct.estType, 'a9')
        %% ESTIMATE

        %-----  Pre-period  -----
        if ~strcmp(momentsUsed, 'post')
            [share_a_pre_hat, share_s_pre_hat, share_l_pre_hat, share_m_pre_hat, ...
                z_pre, P_D_pre, h_0_pre, ...
                W_postSSO_prime_pre, q_postSSO_pre, V_A_pre, V_S_pre, ~, ...
                ~, Q_preSSO_pre, ~, ...
                h_a_pre, h_s_pre, h_a_dist_pre, h_s_dist_pre, ...
                H_pre, H_l_pre, Q_a_pre, Q_s_pre, Q_l_pre] = ...
                    evaluate_objective('pre', params_all, paramsStruct);
        else
            share_a_pre_hat=[]; share_s_pre_hat=[]; share_l_pre_hat=[]; share_m_pre_hat=[];
            z_pre=[]; P_D_pre=[]; h_0_pre=[]; W_postSSO_prime_pre=[];
            q_postSSO_pre=[]; V_A_pre=[]; V_S_pre=[]; Q_preSSO_pre=[];
            h_a_pre=[]; h_s_pre=[]; h_a_dist_pre=[]; h_s_dist_pre=[];
            H_pre=[]; H_l_pre=[]; Q_a_pre=[]; Q_s_pre=[]; Q_l_pre=[];
        end

        %-----  Post-period  -----
        if ~strcmp(momentsUsed, 'pre')
            [share_a_post_hat, share_s_post_hat, share_l_post_hat, share_m_post_hat, ...
                z_post, P_D_post, h_0_post, ...
                W_postSSO_prime_post, q_postSSO_post, V_A_post, V_S_post, V_l_post, ...
                W_preSSO_post, Q_preSSO_post, l_preSSO_post, ...
                h_a_post, h_s_post, h_a_dist_post, h_s_dist_post, ...
                H_post, H_l_post, Q_a_post, Q_s_post, Q_l_post] = ...
                    evaluate_objective('post', params_all, paramsStruct);
        else
            share_a_post_hat=[];  share_s_post_hat=[]; share_l_post_hat=[]; share_m_post_hat=[]; 
            z_post=[]; P_D_post=[]; h_0_post=[]; W_postSSO_prime_post=[];
            q_postSSO_post=[]; V_A_post=[]; V_S_post=[]; V_l_post=[];
            W_preSSO_post=[]; Q_preSSO_post=[]; l_preSSO_post=[]; 
            h_a_post=[]; h_s_post=[]; h_a_dist_post=[]; h_s_dist_post=[];
            H_post=[]; H_l_post=[]; Q_a_post=[]; Q_s_post=[]; Q_l_post=[];
        end

    else
        %% Simulate moments static LOS model

        %pre-PPS moments
        [share_a_pre_hat, share_s_pre_hat, share_m_pre_hat, share_l_pre_hat, ...
         z_pre, P_D_pre, h_0_pre, h_a_pre, h_s_pre, h_a_dist_pre, h_s_dist_pre, ...
         H_pre, H_l_pre]= ...
            static_simulation_forward('pre', params_all, paramsStruct);

        %post-PPS moments
        [share_a_post_hat, share_s_post_hat, share_m_post_hat, share_l_post_hat, ...
         z_post, P_D_post, h_0_post, h_a_post, h_s_post, h_a_dist_post, h_s_dist_post, ...
         H_post, H_l_post]= ...
            static_simulation_forward('post', params_all, paramsStruct);
        
        %placeholder for objects not used in static model
        W_postSSO_prime_pre=[];
        q_postSSO_pre=[]; V_A_pre=[]; V_S_pre=[]; Q_preSSO_pre=[];
        Q_a_pre=[]; Q_s_pre=[]; Q_l_pre=[];
        W_postSSO_prime_post=[];
        q_postSSO_post=[]; V_A_post=[]; V_S_post=[]; V_l_post=[];
        W_preSSO_post=[]; Q_preSSO_post=[]; l_preSSO_post=[]; 
        Q_a_post=[]; Q_s_post=[]; Q_l_post=[];
        
    end
    

    %% Estimate post-LTCH payments 
    if strcmpi(paramsStruct.objType, 'discShares_nonltchPay')
        [nonltchPay_a_pre_hat,  nonltchPay_s_pre_hat, ...
         nonltchPay_a_post_hat, nonltchPay_s_post_hat] ...
                = estimate_post_ltch_payments(nonltchPayModel, ...
            z_pre, z_post, h_a_dist_pre, h_s_dist_pre, h_a_dist_post, h_s_dist_post, ...
            params_all, paramsStruct);

    else
        nonltchPay_a_pre_hat=[];  
        nonltchPay_s_pre_hat=[];
        nonltchPay_a_post_hat=[]; 
        nonltchPay_s_post_hat=[];
    end
        
    %% stack moments
    %pre-PPS post-LTCH payments will be [] if those features not turned on
     moments_hat = ...
         [share_a_pre_hat;  share_s_pre_hat;  share_m_pre_hat;
          share_a_post_hat; share_s_post_hat; share_m_post_hat; ...
          nonltchPay_a_pre_hat;  nonltchPay_s_pre_hat; ...
          nonltchPay_a_post_hat; nonltchPay_s_post_hat];
end

