function jacMat = compute_objfunc_jacobian(...
    objFunc, params_all_hat, paramsDelta)
    % For our chosen objective function (objFunc), compute the Jacobian matrix
    % evaluated at the estimated parameter values.



    % Evaluate the objective function at the current optimal params values.
    [sse_star, temp_paramsStruct] = feval(objFunc, params_all_hat);
    moments_hat_star = temp_paramsStruct.moments_hat_jacobian;

    fprintf('--- Current obj func value: %.4f ---\n', sse_star);

    % Initialize Jacobian, dimension: R X K.
    % R: dimension of moments
    % K: dimension of params vector
    R = length(moments_hat_star);
    K = length(params_all_hat);
    jacMat = 9999*ones(R,K);

    % Perturb each parameter and evaluate first partial derivative.
    for k = 1:K
        delta = paramsDelta(k);
        delta2Inv = 1/(2*delta);
        paramsPerturb_ub = params_all_hat;
        paramsPerturb_ub(k) =  params_all_hat(k) + delta;
        paramsPerturb_lb = params_all_hat;
        paramsPerturb_lb(k) =  params_all_hat(k) - delta;

        % Re-simulate model.
        [~, temp_paramsStruct] = feval(objFunc, paramsPerturb_ub);
        moments_hat_ub = temp_paramsStruct.moments_hat_jacobian;

        [~, temp_paramsStruct] = feval(objFunc, paramsPerturb_lb);
        moments_hat_lb = temp_paramsStruct.moments_hat_jacobian;

        % Populate the Jacobian.
        jacMat(:,k) = (moments_hat_ub - moments_hat_lb) .* delta2Inv;       
    end


end


