function  [cfParamsStruct, cfRetStruct, cf_scenarios] = ...
    counterfactuals(paramsStruct, retStruct)

    fprintf('-- Generate counterfactuals. \n\n');
    close all;

    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % COUNTERFACTUALS (cf)
    % under different payment schedules
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % <<< TT = 120 ("infinite") >>>
    % Initialize a counterfactual returned objects struct as well as
    % counterfactual scenarios set up
    [cfRetStruct,cfParamsStruct, cf_scenarios, scenariosAll, scenariosPlot] = ...
        initialize_cfStruct(retStruct, paramsStruct);

    % Creates payment schedules
    close all;
    [cfRetStruct, cf_scenarios] = create_counterfactual_payment_schedules(...
        cfParamsStruct, cfRetStruct, cf_scenarios);
       
    for scenarioID = scenariosAll
       % Generate counterfactual outcomes
        T_cf = cf_scenarios(scenarioID,'T').T;
        cfRetStruct = run_counterfactual(cell2mat(scenarioID), cfParamsStruct, cfRetStruct,T_cf);
    end


    %% <<< TT = 120 ("infinite") >>>
    % Calculate counterfactual Medicare payments, "infinite" horizon.

    %***** Counterfactual Discharges: Behavioral Effects *****
    cfRetStruct = calculate_counterfactual_payments_allCF(...
        'Cfd', cfParamsStruct, cfRetStruct,scenariosAll, cf_scenarios);

    %***** All Baseline Discharges *****
    cfRetStruct = calculate_counterfactual_payments_allCF(...
        'Bld', cfParamsStruct, cfRetStruct,scenariosAll, cf_scenarios);

    % plot post-discharge mortality under each counterfactual
    calculate_post_mort(30, cfParamsStruct, cfRetStruct, scenariosAll);


    %% generate efficient frontier of LGRI counterfactuals
    % Counterfactual outcomes
    dischargePat='Cfd';
    [cfRetStruct_LGRI, cf_scenarios] = plot_counterfactuals_continuum(...
        dischargePat, cfParamsStruct, cfRetStruct, cf_scenarios);
    
    % Plot "frontier"
    cfRetStruct_LGRI = plot_counterfactuals_continuum_frontier(...
        dischargePat, cfParamsStruct, cfRetStruct_LGRI);

    %% compare across all counterfactuals

    % Select scenarios
    cfRetStruct_LGRI.legendText = cf_scenarios.scenarioFull(scenariosPlot);

    % Export all counterfactual scedules
    if strcmpi(cfParamsStruct.version, 'v18_a3_pool')
        exportxls_payment_schedules(cf_scenarios.scenarioID, cfParamsStruct, cfRetStruct_LGRI);
    end

    if ~strcmp(paramsStruct.estType, 'a9')
        % Plot counterfactual policy functions.
        flag_outputPoliFuncPlot = false;
        for scenarioID = scenariosPlot
            cfRetStruct_LGRI = plot_policy_function('post', cell2mat(scenarioID), ...
                cfParamsStruct, cfRetStruct_LGRI, flag_outputPoliFuncPlot, cf_scenarios);
        end

        % Overlay the policy functions from different payment schedules
        plot_policy_function_overlaid(scenariosPlot, 'post', ...
            cfParamsStruct, cfRetStruct_LGRI);
    end
    
    % Plot counterfactual discharge shares by discharge destination
    % and export results
    for destID = {'a', 's', 'm'}
        plot_counterfactuals_by_destination(scenariosPlot, ...
            cell2mat(destID), cfParamsStruct, cfRetStruct_LGRI);
    end

    close all;
end











