function retStruct_out = est_nonltch_payments_and_health(destID, ...
    period, paramsStruct, retStruct)

    %model specification
    nonltchPayModel = paramsStruct.nonltchPayModel;

    % Initialize structure to return as output.
    retStruct_out = retStruct;

    %destination + period tag in field names
    fldTag = [destID, '_', period];

    % Local variables.
    aggMat = paramsStruct.(['nonLtchPayAggMat_', fldTag]);

    %different aggregation if we consider quadratic payment specification
    payField = ['nonLtchPayAggMatRev_',fldTag];
    if isfield(paramsStruct, payField)
        aggMatPay = paramsStruct.(payField);
    else
        aggMatPay = aggMat;
    end

    % Load non-LTCH moments
    % By day
    byday_nonltchPay  = paramsStruct.(['nonltchPay_'  fldTag]);
    byday_nonltchDays = paramsStruct.(['nonltchDays_' fldTag]);

    % Aggregate based on temp_aggMat.
    nonltchPay   = aggMatPay * byday_nonltchPay;
    nonltchDays  = aggMat    * byday_nonltchDays;

    clear temp_matfile;

    % Define required variables that are local to this function.
    healthDist = retStruct.(['h_',destID, '_dist_', period]);
    z          = retStruct.(['z_',period]);
    
    %% ++++++ post-LTCH payments vs. health model ++++++
    if strcmpi(paramsStruct.objType,'discShares')
        % Non-linear model, estimated using GMM.
        % nonltchPay = zeta0 * exp(zeta1 * health)
        % So E[nonltchPay] = zeta0 * E[exp(zeta1 * healthDist)]
        % OR: ln(nonltchPay) = ln(zeta0) + zeta1 * healthDist
        zeta_0  = paramsStruct.(['zeta_', fldTag, '_0']);
        [zeta, nonltchPay_hat] = ...
            nonlinear_postltch_model(destID, period, 'payments', ...
            nonltchPay, healthDist, zeta_0, ...
            z, paramsStruct, aggMatPay);
    elseif strcmpi(paramsStruct.objType,'discShares_nonltchPay')
        %------------------------------------------
        % For the version of objective function that matches the post-LTCH payments
        % moments as well, we can just extract the estimated zetas from
        % retStruct. We only care about nlgmm in v18.
        % However, we need to estimate post-LTCH days after the model estimation.
        %------------------------------------------
        % The zeta vector in retStruct has already been scaled
        % appropriately (back to levels).
        zeta  = retStruct.(['params_zeta_', fldTag]);
        % Predicted payments by day, which depend on the average health
        % of the discharged patients.
        nonltchPay_hat  = retStruct.(['nonltchPay_', fldTag, '_hat']);
    else
        fprintf('Error: ''%s'' is not a valid argument objType.\n', paramsStruct.objType);
        return
    end

    %% --- Post-LTCH days ---
    % Non-linear model, estimated using GMM.
    coefDays_0 = paramsStruct.(['coefDays_', fldTag, '_0']); 
    [coefDays, nonltchDays_hat] = ...
        nonlinear_postltch_model(destID, period, 'days', ...
        nonltchDays, healthDist, coefDays_0, ...
        z, paramsStruct, aggMat);
  

    %%
    % Update structure to output.
    fldMdlTag = [fldTag '_' nonltchPayModel];

    retStruct_out.(['nonltchPay_'      fldMdlTag]) = nonltchPay;
    retStruct_out.(['zeta_'            fldMdlTag]) = zeta;
    retStruct_out.(['nonltchPay_hat_'  fldMdlTag]) = nonltchPay_hat; 

    retStruct_out.(['nonltchDays_'     fldMdlTag]) = nonltchDays_hat;
    retStruct_out.(['coefDays_'        fldMdlTag]) = coefDays;
    retStruct_out.(['nonltchDays_hat_' fldMdlTag]) = nonltchDays_hat;



return

