%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script runs optimization
% estimate_model -> objective_function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


version = paramsStruct.version;
run = paramsStruct.run;
fprintf('%s -- Version=%s, run=%s: Estimate model. \n\n',char(datetime('now')),version,run);


%% Optimization
opt_MaxFunEvals = 8e4;
opt_MaxIter = 8e4;
opt_TolFun  = 1e-3;
opt_TolX    = 1e-3;

tic
% Set up correct objective function
% estimate both pre and post PPS, just pre, or just post
switch(paramsStruct.momentsUsed)
    case('pre_post')
        objFunc = @(params_all)objective_function(params_all, moments, W_0, paramsStruct);
    case('pre')
        objFunc = @(params_all)objective_function(params_all, moments_pre, W_0_pre, paramsStruct);
    case('post')
        objFunc = @(params_all)objective_function(params_all, moments_post, W_0_post, paramsStruct);
    otherwise
        fprintf('Invalid momentsUsed: %s.\n', momentsUsed);
        return
end


% Optimization options.
options = optimset('Disp', 'iter', 'MaxFunEvals', opt_MaxFunEvals, ...
    'MaxIter', opt_MaxIter, 'TolFun', opt_TolFun, 'TolX', opt_TolX);

if paramsStruct.maxlab > 1
    % Run par_fminsearch - specify maxlab in main before calling this script.
    fprintf('%s -- Optimizer: par_fminsearch \n\n', char(datetime('now')));
    [params_all_hat, sse_hat, exitflag] = par_fminsearch(objFunc, ...
        params_all_0, ...               % Has both pre- and post-period health params + pref params. 
        options, paramsStruct.maxlab);  % Has maxlab.
else
    fprintf('%s -- Optimizer: fminsearch \n\n', char(datetime('now')));
    [params_all_hat, sse_hat, exitflag] = fminsearch(objFunc, ...
        params_all_0, ...               % Has both pre- and post-period health params + pref params. 
        options);                       % No maxlab.
end
fprintf('%s -- End of optimization; sse_hat=%f, exitflag=%d.\n\n',char(datetime('now')),sse_hat,exitflag);
toc


%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STANDARD ERROR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmp(paramsStruct.momentsUsed, 'pre_post') && strcmp(paramsStruct.objType, 'discShares_nonltchPay')
    
    % First get the Jacobian.
    jacMat = compute_objfunc_jacobian(objFunc, params_all_hat, paramsStruct.paramsDelta);
    
    % Now get standard errors.
    [gmmSE, gmmVar] = compute_std_errors(paramsStruct, jacMat);
    
    paramsStruct.jacMat = jacMat;
    paramsStruct.gmmSE  = gmmSE;
    paramsStruct.gmmVar = gmmVar;
    
    fprintf('Finished computing standard errors.\n');
end


%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COLLECT ESTIMATION IN RETSTRUCT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

retStruct = store_estimation_results(paramsStruct, retStruct, ...
                params_all_hat, sse_hat, exitflag);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NON-LTCH PAYMENTS FOR DISCHARGEES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Separately estimate parameters related to non-LTCH payments
% and save them in the already-existing retStruct.
retStruct = postltch_estimation(paramsStruct, retStruct);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARAMETERS TO RECORD to XLS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('\n\n\n')
fprintf('Please record these parameter estimates: \n')
fprintf('LTCH health and preference parameters: params_all_hat \n')
if strcmpi(paramsStruct.objType, 'discShares_nonltchPay') && strcmpi(paramsStruct.version, 'v18_a3_pool')
    fprintf('LTCH health and preference std error: params_all_SE \n')    
    params_all_SE = retStruct.gmmSE;
end
if ~strcmpi(paramsStruct.objType, 'discShares_nonltchPay') 
    fprintf('post-LTCH payment: params_zeta_hat \n')
    params_zeta_hat = [retStruct.zeta_a_post_nlgmm, retStruct.zeta_s_post_nlgmm];
else
    fprintf('post-LTCH days: params_coefDays_hat \n')
    params_coefDays_hat = [retStruct.coefDays_a_post_nlgmm, retStruct.coefDays_s_post_nlgmm];    
end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SAVE WORKSPACE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% delete variables prefixed by temp
out_file = fullfile(projroot, 'Workspace', paramsStruct.outMat);
save(out_file);
fprintf('Saved: %s.\n\n', out_file);

%display parameter estimates
display_params(paramsStruct.paramsEstimated, params_all_hat, params_all_0, ...
                paramsStruct.version, paramsStruct.run, sse_hat, exitflag)

