% LTCH value function and simulate moments
function [share_a_hat, share_s_hat, share_l_hat, share_m_hat, ...
        z, PD, h0, W, q_postSSO, V_A, V_S, V_l, ...
        W_preSSO, Q_preSSO, l_preSSO, ...
        h_a, h_s, h_a_dist, h_s_dist, ...
        H, H_l, Q_a, Q_s, Q_l] = evaluate_objective(period, params_all, paramsStruct)

    % cost and payment
    p_postSSO_pre  = paramsStruct.p_postSSO_pre;
    p_preSSO_post  = paramsStruct.p_preSSO_post;
    p_postSSO_post = paramsStruct.p_postSSO_post;
    cost_pre  = paramsStruct.repCostPerDay_pre;
    cost_post = paramsStruct.repCostPerDay_post;

    % timing
    n   = paramsStruct.n;
    backInductT   = paramsStruct.backInductT; % The day when health becomes a stationary process


    %key LTCH parameters
    [params_health_pre, params_health_post, params_pref, param_scale, ~, ~, ~, ~] ...
              = assign_params(params_all, paramsStruct);

    %-----  Pre-period  -----
    if strcmp(period, 'pre')
        % Health process: estimate surival rate and mortality hazard. 
        [z, PD, h0] = health_process(params_health_pre, paramsStruct);
        % Post-SSO discharge value function iteration
        [W, q_postSSO, V_A, V_S, V_l] = ...
            post_value_function(params_pref, param_scale, cost_pre, ...
            z,PD, p_postSSO_pre, paramsStruct);
        % The value function is stationary in the pre-PPS period. Thus, we can 
        % eliminate the pre-value function in this period. However, we need to
        % construct the appropriate Q_preSSO_pre matrix to feed into simulate_forward.
        Q_preSSO=reshape(ones(backInductT-1,1)*q_postSSO(:)',backInductT-1,n,3);
        W_preSSO=[];l_preSSO=[];

    end

    %-----  Post-period  -----
    if strcmp(period, 'post')
        % Health process: estimate surival rate and mortality hazard. 
        [z, PD, h0] = health_process(...
            params_health_post, paramsStruct);
        % Post-SSO discharge value function iteration
        [W, q_postSSO, V_A, V_S, V_l] = ...
            post_value_function(params_pref, param_scale, cost_post, ...
            z,PD,p_postSSO_post, paramsStruct);
        % Pre-SSO discharge value function
        [W_preSSO, Q_preSSO, l_preSSO] = pre_value_function(...
            params_pref, param_scale, cost_post, ...
            W, z, PD, p_preSSO_post, paramsStruct);
    end

    %-----  Simulate forward  -----
    [share_a_hat, share_s_hat, share_l_hat, share_m_hat, ...
        h_a, h_s, h_a_dist, h_s_dist, ...
        H, H_l, Q_a, Q_s, Q_l] = simulate_forward(...
        q_postSSO, Q_preSSO, z, PD, h0, paramsStruct);
end