function [z,P_D_struct,h_0] = health_process(params_health,paramsStruct)


% Estimate surival rate and mortality hazard. 


% number of standard deviation does health grid span across
num_sd = paramsStruct.healthGridNumSdev;

%params health
if strcmp(paramsStruct.estType, 'a5')
    mu    = params_health(1);
    rho   = params_health(2);
    rho1  = params_health(3);
    sigma = params_health(4);
    mu0    = params_health(5);
    sigma0 = params_health(6);    
else
    mu    = params_health(1);
    rho   = params_health(2);
    sigma = params_health(3);
    mu0   = params_health(4);
    sigma0 = params_health(5);
    rho1 = 0;    
end

n   = paramsStruct.n;
backInductT   = paramsStruct.backInductT; % The day when health becomes a stationary process

% Initialize the full (n X n) transition probability matrix (P_D) as a struct. Each
% element of that struct is a trans prob matrix corresponding to a certain
% day (t). We are okay with overwriting the trans prob matrix without death
% (n-1 X n-1) in each loop of t.
P_D_struct = struct();

% Initialize empty z and S for t==1. When t gets incremented, we will have
% non-empty z and S from t==1.
z = [];
S = [];

%%
% Determine if we need to loop over t to create time-varying health
% processes. endLoopT==1 means just one loop!
if strcmp(paramsStruct.estType, 'a5')
    endLoopT = backInductT;
else
    endLoopT = 1;
end
 

for t=1:1:endLoopT
    % Create the time-varying "rho". This is just rho when t==1.
    time_var_rho = rho + rho1*log(t);

    %transition prob matrix: P is the transition prob matrix for alive patients.
    [z, P] = markovprob(mu,time_var_rho,sigma,n-1,num_sd, z, paramsStruct);

    % Survival probability. We only need to calculate this once since we
    % determine z only once, i.e. when t==1.
    if t==1
        S = normcdf(z');
    end
    
    %add death to transition prob matrix as aborbing state:
    % P_D is P but with an additional state, which is death (absorbing). This
    % means we need to scale the transition probs for alive states down.
    P_D_t = zeros(n,n);
    P_D_t(:,1) = [1; 1-S];

    %scale down other transitions by prob of survival
    for i = 1:n-1
        P_D_t(2:n,i+1) = P(:,i).*S;
    end

    % Save the single P_D as an entry of the struct. Assign it as P_D_(backInductT)
    % if we don't have a time-varying health process.
    if endLoopT==1
        P_D_struct.(['P_D_' num2str(backInductT)])= P_D_t;
    else
        P_D_struct.(['P_D_' num2str(t)])= P_D_t;
    end
end

%%
%initial distribution
h_0 = zeros(n-1,1);
w = z(n-1) - z(n-2); % Width between grid points.

h_0(1) = normcdf((z(1) + w/2 - mu0)/sigma0);
h_0(n-1) = 1 - normcdf((z(n-1) - w/2 - mu0)/sigma0);
for j = 2:n-2,
    h_0(j) = normcdf((z(j) + w/2 - mu0)/sigma0) - ...
        normcdf((z(j) - w/2 - mu0)/sigma0,0,1);
end