%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The main code to import raw data and save them as MATLAB objects. 
% This code works in parallel with main_a.m.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear variables; clc

global projroot dataroot subdatadir log_file out_dir;
global grandver;


% Set up directories  
projroot = '<project directory>';
dataroot = '<data directory>';


log_file = fullfile(dataroot,'import_data.log');
diaryStatus = get(0,'Diary'); diaryFileName = get(0,'DiaryFile');
% start logging
if not(strcmp(diaryStatus,'on') && strcmp(log_file,diaryFileName))
    diary off;
    diary(log_file);
    diaryStatus = get(0,'Diary'); diaryFileName = get(0,'DiaryFile');
end

cd(fullfile(projroot, 'Code'));
fprintf('%s -- My current directory is \n%s.\n\n',char(datetime('now')),pwd);
out_dir = fullfile(projroot, 'Output', filesep);    % output directory


grandver = 'v17';   % the grand version that identifies 
% Repository of various structs.
repoMatFile = fullfile(dataroot, ['repo_param_and_returned_structs_' grandver '.mat']);

%% ---------------------------------------------------------------------%%
% Target moments: combine pre- and post-PPS moments into the same object.
%%----------------------------------------------------------------------%%
subdatadir = 'From_JIRA/LTCH-53';
xlsFile = fullfile(dataroot, subdatadir, 'pre_post_graphdata.xls');
temp_importedNum = xlsread(xlsFile, 'hist_los_dest_hi_no_wt', 'B2:C62');
share_a_pre = temp_importedNum(:,1);    % Extended: -15 to 45
share_a_post = temp_importedNum(:,2);	% Extended: -15 to 45

temp_importedNum = xlsread(xlsFile, 'hist_los_dest_lo_no_wt', 'B2:C62');
share_s_pre = temp_importedNum(:,1);
share_s_post = temp_importedNum(:,2);

temp_importedNum = xlsread(xlsFile, 'hist_los_dest_death_no_wt', 'B2:C62');
share_m_pre = temp_importedNum(:,1);
share_m_post = temp_importedNum(:,2);

target_moments_src = 'pooled';
out_file = fullfile(dataroot, 'target_pool');
save(out_file, '-regexp', '^share_', '^target_moments_src$');

clear temp_* share_* target_moments_src;
subdatadir = '';

%%
subdatadir = 'From_JIRA/LTCH-53';
xlsFile = fullfile(dataroot, subdatadir, 'pre_post_graphdata_fp_np.xls');
temp_importedNum = xlsread(xlsFile, 'hist_los_dest_hi_fp_np', 'B2:C62');
share_a_pre = temp_importedNum(:,1);    % Extended: -15 to 45
share_a_post = temp_importedNum(:,2);	% Extended: -15 to 45

temp_importedNum = xlsread(xlsFile, 'hist_los_dest_lo_fp_np', 'B2:C62');
share_s_pre = temp_importedNum(:,1);
share_s_post = temp_importedNum(:,2);

temp_importedNum = xlsread(xlsFile, 'hist_los_dest_death_fp_np', 'B2:C62');
share_m_pre = temp_importedNum(:,1);
share_m_post = temp_importedNum(:,2);

target_moments_src = 'fp';
out_file = fullfile(dataroot, 'target_fp');
save(out_file, '-regexp', '^share_', '^target_moments_src$');

clear temp_* share_* target_moments_src;
subdatadir = '';

%%
subdatadir = 'From_JIRA/LTCH-53';
xlsFile = fullfile(dataroot, subdatadir, 'pre_post_graphdata_fp_np.xls');
temp_importedNum = xlsread(xlsFile, 'hist_los_dest_hi_fp_np', 'D2:E62');
share_a_pre = temp_importedNum(:,1);    % Extended: -15 to 45
share_a_post = temp_importedNum(:,2);	% Extended: -15 to 45

temp_importedNum = xlsread(xlsFile, 'hist_los_dest_lo_fp_np', 'D2:E62');
share_s_pre = temp_importedNum(:,1);
share_s_post = temp_importedNum(:,2);

temp_importedNum = xlsread(xlsFile, 'hist_los_dest_death_fp_np', 'D2:E62');
share_m_pre = temp_importedNum(:,1);
share_m_post = temp_importedNum(:,2);

target_moments_src = 'npgov';
out_file = fullfile(dataroot, 'target_npgov');
save(out_file, '-regexp', '^share_', '^target_moments_src$');

clear temp_* share_* target_moments_src;
subdatadir = '';

%% ---------------------------------------------------------------------%%
% Post-Discharge from LTCH Payments: Pooled, Post-PPS
%%----------------------------------------------------------------------%%
subdatadir = 'From_JIRA/LTCH-58';
[~, ~, temp_importedRaw] = xlsread(fullfile(dataroot, subdatadir, ...
    'post_dschrg_stats_post.xlsx'), 'stats_by_los_Dest','A2:M123');
temp_importedRaw(cellfun(@(x) ~isempty(x) && isnumeric(x) && isnan(x),temp_importedRaw)) = {''};
temp_cellVectors = temp_importedRaw(:,2);
temp_data = temp_importedRaw(:,[1,3:end]);
% Transform cells containing numeric data to numeric marices.
temp_data = reshape([temp_data{:}],size(temp_data));

% Allocate imported array to column variable names
temp_dest = temp_cellVectors(:,1);

price_a_ub = temp_data(strcmpi(temp_dest,'upstream'),2);
price_a_lb = temp_data(strcmpi(temp_dest,'upstream'),3);
los_a_ub = temp_data(strcmpi(temp_dest,'upstream'),4);
los_a_lb = temp_data(strcmpi(temp_dest,'upstream'),5);
mort60_a = temp_data(strcmpi(temp_dest,'upstream'),6);
ln_price_a_ub = temp_data(strcmpi(temp_dest,'upstream'),7);
ln_price_a_lb = temp_data(strcmpi(temp_dest,'upstream'),8);
ln_los_a_ub = temp_data(strcmpi(temp_dest,'upstream'),9);
ln_los_a_lb = temp_data(strcmpi(temp_dest,'upstream'),10);
ln_mort60_a = temp_data(strcmpi(temp_dest,'upstream'),11);
post_ltch_N_a = temp_data(strcmpi(temp_dest,'upstream'),12);

price_s_ub = temp_data(strcmpi(temp_dest,'downstream'),2);
price_s_lb = temp_data(strcmpi(temp_dest,'downstream'),3);
los_s_ub = temp_data(strcmpi(temp_dest,'downstream'),4);
los_s_lb = temp_data(strcmpi(temp_dest,'downstream'),5);
mort60_s = temp_data(strcmpi(temp_dest,'downstream'),6);
ln_price_s_ub = temp_data(strcmpi(temp_dest,'downstream'),7);
ln_price_s_lb = temp_data(strcmpi(temp_dest,'downstream'),8);
ln_los_s_ub = temp_data(strcmpi(temp_dest,'downstream'),9);
ln_los_s_lb = temp_data(strcmpi(temp_dest,'downstream'),10);
ln_mort60_s = temp_data(strcmpi(temp_dest,'downstream'),11);
post_ltch_N_s = temp_data(strcmpi(temp_dest,'downstream'),12);

out_file = fullfile(dataroot, 'nonLtchPay_post_pool');
save(out_file, '-regexp', 'post_ltch|price_|los_|mort60');

clear temp_* *post_ltch_* *mort60* *_ub* *_lb*;
subdatadir = '';

%% ---------------------------------------------------------------------%%
% Post-Discharge from LTCH Payments: Pooled, Pre-PPS
%%----------------------------------------------------------------------%%
subdatadir = 'From_JIRA/LTCH-58';
[~, ~, temp_importedRaw] = xlsread(fullfile(dataroot, subdatadir, ...
    'post_dschrg_stats_pre.xlsx'), 'stats_by_los_Dest','A2:M123');
temp_importedRaw(cellfun(@(x) ~isempty(x) && isnumeric(x) && isnan(x),temp_importedRaw)) = {''};
temp_cellVectors = temp_importedRaw(:,2);
temp_data = temp_importedRaw(:,[1,3:end]);
% Transform cells containing numeric data to numeric marices.
temp_data = reshape([temp_data{:}],size(temp_data));

% Allocate imported array to column variable names
temp_dest = temp_cellVectors(:,1);

price_a_ub = temp_data(strcmpi(temp_dest,'upstream'),2);
price_a_lb = temp_data(strcmpi(temp_dest,'upstream'),3);
los_a_ub = temp_data(strcmpi(temp_dest,'upstream'),4);
los_a_lb = temp_data(strcmpi(temp_dest,'upstream'),5);
mort60_a = temp_data(strcmpi(temp_dest,'upstream'),6);
ln_price_a_ub = temp_data(strcmpi(temp_dest,'upstream'),7);
ln_price_a_lb = temp_data(strcmpi(temp_dest,'upstream'),8);
ln_los_a_ub = temp_data(strcmpi(temp_dest,'upstream'),9);
ln_los_a_lb = temp_data(strcmpi(temp_dest,'upstream'),10);
ln_mort60_a = temp_data(strcmpi(temp_dest,'upstream'),11);
post_ltch_N_a = temp_data(strcmpi(temp_dest,'upstream'),12);

price_s_ub = temp_data(strcmpi(temp_dest,'downstream'),2);
price_s_lb = temp_data(strcmpi(temp_dest,'downstream'),3);
los_s_ub = temp_data(strcmpi(temp_dest,'downstream'),4);
los_s_lb = temp_data(strcmpi(temp_dest,'downstream'),5);
mort60_s = temp_data(strcmpi(temp_dest,'downstream'),6);
ln_price_s_ub = temp_data(strcmpi(temp_dest,'downstream'),7);
ln_price_s_lb = temp_data(strcmpi(temp_dest,'downstream'),8);
ln_los_s_ub = temp_data(strcmpi(temp_dest,'downstream'),9);
ln_los_s_lb = temp_data(strcmpi(temp_dest,'downstream'),10);
ln_mort60_s = temp_data(strcmpi(temp_dest,'downstream'),11);
post_ltch_N_s = temp_data(strcmpi(temp_dest,'downstream'),12);

out_file = fullfile(dataroot, 'nonLtchPay_pre_pool');
save(out_file, '-regexp', 'post_ltch|price_|los_|mort60');

clear temp_* *post_ltch_* *mort60* *_ub* *_lb*;
subdatadir = '';



%% ---------------------------------------------------------------------%%
% Variance-Covariance Matrix for GMM Standard Errors
%%----------------------------------------------------------------------%%
% SEs, 
subdatadir = 'From_JIRA/SEs';

%%
% Import the data: POOLED
% Rows 1-366: discharges (61 days * 3 dest * 2 periods)
% 367-370: Post-LTCH payments, pre-PPS
% 371-374: Post-LTCH payments, post-PPS
bootstrap_var_m_hat = xlsread(fullfile(dataroot,subdatadir,'wtd_cov.xlsx'),...
    'wtd_cov','A1:NJ374');

out_file = fullfile(dataroot, 'varcovMatrix_pool');
save(out_file, 'bootstrap_var_m_hat');

%%
% One version of model a3 drops the pre-PPS post-LTCH moments, so we need
% the variance-covariance matrix to drop those too.
% POOLED
bootstrap_var_m_hat(367:370,:) = [];
bootstrap_var_m_hat(:,367:370) = [];

out_file = fullfile(dataroot, 'varcovMatrix_pool_omitPrePPSNonLtch');
save(out_file, 'bootstrap_var_m_hat');

% SEs
subdatadir = 'From_JIRA/SEs';

%%
diary off;