%% Initialize counterfactual parameters and returned objects struct.
function [cfRetStruct,cfParamsStruct, cf_scenarios, scenariosAll, scenariosPlot] = ...
            initialize_cfStruct(retStruct, paramsStruct)

    cfRetStruct = retStruct;

    % Initialize a counterfactual params struct.
    cfParamsStruct = paramsStruct;

    % Modify key params for counterfactuals.
    cfParamsStruct.n_cf     = paramsStruct.n;
    % T is SSO day
    cfParamsStruct.T_cf     = paramsStruct.T;
    % backInductT is the day the health process becomes stationary.
    % IMPORTANT: backInductT >= T in the observed schedule!!!
    cfParamsStruct.backInductT_cf     = paramsStruct.backInductT;
    cfParamsStruct.TT_cf    = 120;
    cfParamsStruct.days_cf  = (1 - cfParamsStruct.T_cf ):1:(cfParamsStruct.TT_cf - cfParamsStruct.T_cf );                

    
    %%
    % Create a table from a cell array containing counterfactual info.
    % The table's row names are the scenario IDs.
    % Columns (in order): scenarioID full_name include scenario-specific-T
    cf_scenarios = {...
        'baseline'  'Observed Schedule' ;...
        'cons'      'Constant Per-Diem' ;...
        'consCap'   'Extended SSO'      ;...
        'lower'     'Lower Cap'         ;...
        'higher'    'Higher Per Day'    ;...
        'sPay'      'Constant Per-Diem (SNF Amount)'     ;...
        'estCost'   'Constant Per-Diem (Estimated Cost)'     ;...
        'sPayCap'   'Constant Per-Diem (SNF Amount) with Cap'     ;...
        'estCostCap'   'Constant Per-Diem (Estimated Cost) with Cap'     ...
        };
    cf_scenarios = cell2table(cf_scenarios, ...
        'variablenames', {'scenarioID','scenarioFull'});
    cf_scenarios.Properties.RowNames = cf_scenarios.scenarioID;
    cf_scenarios.include = ones(numel(cf_scenarios.scenarioID),1);
    cf_scenarios.include('cons') = 0; % exclude the constant per-diem scenario
    
    
    %% determine scenarios to run and to plot
    scenariosAll  = {'baseline', 'cons', 'consCap', 'lower', 'higher', 'sPayCap', 'estCostCap'};   
    scenariosPlot = {'baseline', 'higher', 'lower', 'LGRI_17', 'sPayCap', 'estCostCap'};
end