%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Grand versions:
% - v17: Use updated moments and data for params, use log sum formula in 
% pre-value function, allow for restriction of health process to only vary
% by initial values, and allow for the estimation of the scale parameter.
% - v18: All of changes introduced in v17, plus adding the post-discharge
% (from LTCH) payments as moments to match in the objective function.
%
% Usage note:
% - This driver file calls other MATLAB scripts and functions to do all the
% major operations for the estimation part of this project.
% - You must always run this section of the driver file, which does the
% setup.
% - Before estimating a model's parameters, make sure that you have the
% correct initial values in setup_params(),
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear variables; clc; close all;



% Note:
% pre vs. post = pre-PPS vs. post-PPS
% preSSO vs. postSSO = pre-SSO-threshold vs. post-SSO-threshold

% Set up directories 
projroot = '<project directory>';
dataroot = '<data directory>';

cd(fullfile(projroot, 'Matlab'));
fprintf('%s -- My current directory is \n%s.\n\n',char(datetime('now')),pwd);
seed = 1009;        % set seed for simulation


%****************************************
% version and run tag
grandver = 'v17';

% Basecase        : _a3_pool  : v17 run 13, v18 run 11
% time-varying rho: _a5_pool  : v17 run 9 , v18 run 5
% post-PPS only   : _a3_d_pool: v17 run 5 , v18 run 5

% Basecase myopic : _a8_pool  : v17 run 1
% Basecase static : _a9_pool  : v17 run 1

version = [grandver '_a9_pool'];
run = '1'; 

% Initial values
params_all_0_pool = setup_params();

% Set model parameters and global arrays.
set_model_params_and_globals;

%% Estimation
estimate_model;     



%% Plots & Counterfactuals
% for our three dynamic models: a3, a5 and a3_d, we only generate results
% for model v18
% for alternative static model: a_8 and a_9, we plot model fit for model
% v17

load_estimates;

% plot model fit
[paramsStruct, retStruct] = plot_model_results(paramsStruct, retStruct);

%generate and plot counterfactuals
[cfParamsStruct,cfRetStruct,cf_scenarios] = counterfactuals(paramsStruct, retStruct);

%robustness exercise (v18, a3 only)
robustness(paramsStruct, retStruct);

