function [z, P] = markovprob(mu, rho, sigma, n, num_sd, z, paramsStruct)
% Approximate n-state AR(1) process following Tauchen (1986). 
%
% Z(t+1) = mu + rho*Z(t) + eps(t+1)
%
% where  std(eps) = sigma


%%

%trim rho and sigma
% We now additionally implement these constraints as penalties in the
% objective function. However, since sz involves sqrt(1 - rho^2), we still need
% to trim rho here.
rho   = min(max(-0.99999,rho), 0.99999);
sigma = max(0.00001, sigma);

% in some perturbation exercise, we change health parameters, but we don't
% want to have different grid. paramsStruct.fixHGrid store estimated health
% parameters
if isfield(paramsStruct,'fixHGrid')
    muG    = paramsStruct.fixHGrid(1);
    rhoG   = paramsStruct.fixHGrid(2);
    sigmaG = paramsStruct.fixHGrid(3);

    rhoG   = min(max(-0.99999,rhoG), 0.99999);
    sigmaG = max(0.00001, sigmaG);
else   
    muG    = mu;
    rhoG   = rho;
    sigmaG = sigma;
end

% Determine the health grid points. We only create the health grid points
% once at t=1. For the time-varying health process, we just take as given
% the grid points determined on t=1.
if isempty(z)
    sz = sigmaG / ((1-rhoG^2)^(1/2)); % Std dev of z
    zmin = -num_sd * sz + muG/(1-rhoG);
    zmax = num_sd * sz + muG/(1-rhoG);
    z = linspace(zmin,zmax,n); % Grid Points
end


% Transition matrix:
P = zeros(n,n); 
w = z(n) - z(n-1);
wHalfStep = w/2;
for j = 1:n
    P(j,1) = normcdf((z(1) + wHalfStep - mu - rho*z(j))/sigma);
    P(j,n) = 1 - normcdf((z(n) - wHalfStep - mu - rho*z(j))/sigma);
    P(j,2:n-1) = normcdf((z(2:n-1) + wHalfStep - mu - rho*z(j))/sigma) - ...
        normcdf((z(2:n-1) - wHalfStep - mu - rho*z(j))/sigma);
end

