function [coef_hat, postltch_hat] = ...
    nonlinear_postltch_model(destID, period, outcomeType, ....
    postltch_moments, healthDist, coef_0, ...
    z, paramsStruct, aggMat)

    %model specification
    nonltchPayModel = paramsStruct.nonltchPayModel;

    % Use the identity matrix as the weighting matrix for now.
    % We might need the optimal weightMat for efficient GMM later.
    weightMat = eye(numel(postltch_moments)); 

    opt_MaxFunEvals = 1e5;
    opt_MaxIter = 1e5;
    opt_TolFun  = 1e-5;
    opt_TolX    = 1e-5;

    tic
    options = optimset('Disp', 'iter', 'MaxFunEvals', opt_MaxFunEvals, ...
        'MaxIter', opt_MaxIter, 'TolFun', opt_TolFun, 'TolX', opt_TolX);
    % Run fminsearch, no need to input maxlab.
    fprintf('%s -- Optimizer: fminsearch \n\n', char(datetime('now')));
    [coef_hat, sseHat, exitflag] = ...
        fminsearch(@(coef) ...
        nonlinear_nonltch_objfunc(...
            coef, nonltchPayModel, postltch_moments, healthDist, z, ...
            weightMat, aggMat ... 
        ), ...
        coef_0, options); 
    fprintf('%s -- Post-LTCH %s model; sseHat=%f, exitflag=%d.\n\n',...
        char(datetime('now')), outcomeType, sseHat,exitflag);
    toc

    %--------------------------------------------------------------------------
    % Estimated payments by day for the specified facility, based on the 
    % parameters from this non-linear model.
    byday_postltch_hat = nonltch_spec(nonltchPayModel, coef_hat, z, healthDist);
    postltch_hat = aggMat * byday_postltch_hat;


    fprintf(['%s -- destID: %s, period: %s, outcomeType: %s\n', ...
        'coef0_hat=%.2f, coef1_hat=%0.4f.\n\n'],...
        char(datetime('now')), destID, period, outcomeType, coef_hat(1),coef_hat(2));
end
%--------------------------------------------------------------------------
function sseHat = nonlinear_nonltch_objfunc(coef, nonltchPayModel, ...
    postltch_moments, healthDist, z, weightMat, aggMat)

    %--- SSE ---
    postltch = nonltch_spec(nonltchPayModel, coef, z, healthDist);

    modelErr = postltch_moments - aggMat * postltch;
    sseHat = modelErr' * weightMat * modelErr;
end
%--------------------------------------------------------------------------

