function [] = plot_counterfactuals_by_destination(...
    scenariosToPlot, destID, cfParamsStruct, cfRetStruct)

    % Local parameters.
    daysTopCode = 60;
    lineStyles = {'-k','-c','-g','-r','-y','-b'};

    % Define required variables that are local to this function.
    days_cf = cfParamsStruct.days_cf;

    % Get destination name.
    destShort  = set_plot_names(destID);

    temp_colnames = {'days'};
    temp_outmat = days_cf(:);

    for scenarioID = {scenariosToPlot{:}}
        scenarioID = cell2mat(scenarioID);

        temp_share = cfRetStruct.(['share_', destID, '_post_', scenarioID]);

        temp_colnames = [temp_colnames, ['share_' scenarioID]];
        temp_outmat = [temp_outmat, temp_share(:)];

        eval(['share_' scenarioID '= temp_share;']);
    end

    % Make line plots for the selected destination.
    % Each line represents one payment schedule.
    xTickMin = min(days_cf);
    xTickMax = max(days_cf);
    xMin = xTickMin - 0.7;
    xMax = min(xTickMax, daysTopCode) + 0.7; % top-code days

    iter = 1;
    plot_share_str = '';
    for scenarioID = {scenariosToPlot{:}}
        scenarioID = cell2mat(scenarioID);
        lineStyle = cell2mat(lineStyles(iter));
        plot_share_str = [plot_share_str 'days_cf, share_' scenarioID ', ''' lineStyle ''','];
        iter = iter + 1;
    end

    fig_handle = figure; hold on;
    plot_handle = eval(['plot(' plot_share_str '''LineWidth'',2)']);
    hold off;

    title({['Shares discharged to ' destShort ' in the post-period'],...
        'Under actual and counterfactual payment schedules',''},'FontSize',18);
    leg = legend(plot_handle, cfRetStruct.legendText, 'Location', 'northeast'); set(leg, 'FontSize', 10); legend boxoff;
    xlabel('Days','FontSize',18);
    ylabel('Share Discharged','FontSize',18);
    xlim([xMin xMax]);

    ax = gca;
    ax.XTick = xTickMin:5:xTickMax;

    out_file = fullfile(cfParamsStruct.outDir, 'Figures', ...
        ['counterfactual_share_' destID '_post_overlaid_' cfParamsStruct.version '.pdf']);
    print('-painters','-dpdf','-r300',out_file);
    fprintf('Printed %s\n', out_file);

    %% Output data to Excel.
    xlsOutFile = cfParamsStruct.xlsOutFile;
    xlsSheetName = ['cf_share_' destID '_' cfParamsStruct.xlsShtNm];
    xlswrite(xlsOutFile,temp_colnames,xlsSheetName);
    xlswrite(xlsOutFile,temp_outmat,xlsSheetName,'A2');

    %print time stamp
    print_timestamp(xlsOutFile, xlsSheetName, cfParamsStruct.version, cfParamsStruct.run);

return