function [cfRetStruct_out, cf_scenarios] = plot_counterfactuals_continuum(...
    dischargePat, cfParamsStruct, cfRetStruct, cf_scenarios)

%%
% Local parameters.
daysRng = 6:1:26; % Correspond to Days -10 through 10.
nonltchPayModel = cfParamsStruct.nonltchPayModel;

% Initialize the output container.
cfRetStruct_out = cfRetStruct;

% Define required variables that are local to this function.
T_cf = cfParamsStruct.T_cf;
TT_cf = cfParamsStruct.TT_cf;

d_share_l_baseline = cfRetStruct.d_share_l_post_baseline;
revenueLGRI = cfRetStruct.payments_l_post_baseline;

% Convert percentage never discharged from LTCH to decimals.
share_l_never_disc = cfRetStruct.pct_l_never_disc_post_baseline / 100;

% Create payment schedule with kink on each selected day consistent with
% LGRI - use the baseline discharge behavior to get the payment schedule,
% then get the behavioral effects (specified as 'Cfd' as input to dischargePat).
iter = 1;
for capDay = daysRng
    paySchedule = capDay*ones(TT_cf,1);
    capDay_rel_SSO = capDay - T_cf;

    for preCapDay = 1:(capDay-1);
        paySchedule(preCapDay) = preCapDay;
    end
    
    perdiem_preKink = revenueLGRI / ...
        (share_l_never_disc * paySchedule(TT_cf) + d_share_l_baseline' * paySchedule);
    
    p_LGRI = perdiem_preKink * paySchedule;
    eval(['cfRetStruct_out.p_LGRI_' num2str(capDay) ' = p_LGRI;']);
    capAmount = p_LGRI(capDay);
    
    % Create a scenario ID.
    scenarioID = ['LGRI_' num2str(capDay)];
    
    % Split price schdule into pre-SSO vector and post-SSO per diem.
    temp_p_preSSO = ones(capDay,1)*perdiem_preKink;
    eval(['cfRetStruct_out.p_preSSO_' scenarioID '= temp_p_preSSO;']);
    eval(['cfRetStruct_out.p_postSSO_' scenarioID '= 0;']);

    % Update cf_scenarios.
    temp_cells = {'scenarioID', 'scenarioFull', 'include', 'T'; ...
        scenarioID, ['LGRI with New SSO Cap Day on Day ' num2str(capDay_rel_SSO)], 1, capDay};
    temp_table = cell2table(temp_cells(2:end,:));
    temp_table.Properties.VariableNames = temp_cells(1,:);

    % do not insert more rows into the scenario table if this scenario is
    % already there
    if sum(ismember(cf_scenarios.scenarioID, scenarioID))==0
        cf_scenarios = [cf_scenarios; temp_table];
        cf_scenarios.Properties.RowNames = cf_scenarios.scenarioID;
    else
        cf_scenarios(scenarioID, 'T')={temp_table.T};
    end
    
    % Get counterfactual discharges using this alternative payment
    % schedule.
    cfRetStruct_out = run_counterfactual(...
        scenarioID, cfParamsStruct, cfRetStruct_out, ...
        cf_scenarios(scenarioID,'T').T);

    % Get counterfactual discharge shares for this combination of period and
    % scenario, then calculate discharge PDF and hazard.
    share_l_cf = eval(['cfRetStruct_out.share_l_post_' scenarioID]);
    d_share_l_cf = -diff([1; share_l_cf]);                  % discharge PDF
    d_haz_l_cf = d_share_l_cf ./ [1; share_l_cf(1:end-1)];  % discharge hazard

    % Populate cfRetStruct_out with the discharge PDF and hazard, and % of
    % patients never discharged from the LTCH.
    eval(['cfRetStruct_out.d_share_l_post_' scenarioID '= d_share_l_cf;']);
    eval(['cfRetStruct_out.d_haz_l_post_' scenarioID '= d_haz_l_cf;']);

    pct_l_never_disc_cf = (1-sum(d_share_l_cf))*100;
    eval(['cfRetStruct_out.pct_l_never_disc_post_' scenarioID '= pct_l_never_disc_cf;']);

    % Calculate counterfactual Medicare payments.
    [temp_output, cfRetStruct_out] = calculate_counterfactual_payments(...
        scenarioID, dischargePat, cfParamsStruct, cfRetStruct_out);
    temp_output = [temp_output; revenueLGRI; capDay; capDay_rel_SSO;...
        perdiem_preKink; capAmount];
    eval(['output_' scenarioID '= temp_output;']);
    
    if iter ==1
        outputCols = ['output_' scenarioID];
    else
        outputCols = [outputCols ', ' ['output_' scenarioID]];
    end
    iter = iter + 1;
    
    fprintf('Scenario=%s, never discharged=%0.2f%%\n', scenarioID,pct_l_never_disc_cf);
end

clear temp_*;

%%
%------- Assemble table to output.
temp_cf_systemPay_rowNames = {'share_l_not_disc_60days'; 'share_l_never_disc'; 'tot_payments'; ...
    'payments_l'; 'profits_l'; 'alos'; 'payments_l_per_day'; 'profits_l_per_day'; ...
    'tot_payments_a'; 'tot_share_a'; 'avg_payments_a'; 'avg_days_a'; 'avg_health_a'; ...
    'tot_payments_s'; 'tot_share_s'; 'avg_payments_s'; 'avg_days_s'; 'avg_health_s'; ...
    'tot_share_m'; ...
    'tot_share_a_preSSO'; 'tot_share_a_postSSO'; 'avg_health_a_preSSO'; 'avg_health_a_postSSO'; ...
    'tot_payments_a_preSSO'; 'tot_payments_a_postSSO'; ...
    'avg_payments_a_preSSO'; 'avg_payments_a_postSSO'; ...
    'avg_days_a_preSSO'; 'avg_days_a_postSSO'; ...
    'tot_share_s_preSSO'; 'tot_share_s_postSSO'; 'avg_health_s_preSSO'; 'avg_health_s_postSSO'; ...
    'tot_payments_s_preSSO'; 'tot_payments_s_postSSO'; ...
    'avg_payments_s_preSSO'; 'avg_payments_s_postSSO'; ...
    'avg_days_s_preSSO'; 'avg_days_s_postSSO'; ...
    'tot_share_m_preSSO'; 'tot_share_m_postSSO'; ...
    'revenueLGRI'; 'capDay'; 'capDay_rel_SSO'; 'perdiem_preKink'; 'capAmount'};

% Create a table of counterfactual systemwide payments across all scenarios.
temp_cf_systemPay = eval(['table(' outputCols ...
    ', ''RowNames'', temp_cf_systemPay_rowNames)']);
eval(['cfRetStruct_out.cf_systemPay_LGRI_' nonltchPayModel dischargePat '= temp_cf_systemPay;']);

%%  Write table of counterfactual systemwide payments to Excel.
xlsSheetName = ['LGRI', cfParamsStruct.xlsShtNm '_' nonltchPayModel dischargePat];
xlsOutFile = cfParamsStruct.xlsOutFile;

writetable(temp_cf_systemPay, xlsOutFile, 'Sheet', xlsSheetName, ...
    'WriteRowNames',true);

%print time stamp
print_timestamp(xlsOutFile, xlsSheetName, cfParamsStruct.version, cfParamsStruct.run);


%%
return

