function [cfRetStruct_out] = plot_counterfactuals_continuum_frontier(...
    dischargePat, cfParamsStruct, cfRetStruct)

%mdoel specification
nonltchPayModel = cfParamsStruct.nonltchPayModel;

% Initialize the output container.
cfRetStruct_out = cfRetStruct;

% Define required variables that are local to this function.
temp_cf_systemPay = cfRetStruct.(['cf_systemPay_LGRI_' nonltchPayModel dischargePat]);

% Scatter LTCH profits/payments against negative Medicare payments.
profits_l = cell2mat(table2cell(temp_cf_systemPay('profits_l',:)));
payments_l = cell2mat(table2cell(temp_cf_systemPay('payments_l',:)));
tot_payments = cell2mat(table2cell(temp_cf_systemPay('tot_payments',:)));
capDay_rel_SSO = cell2mat(table2cell(temp_cf_systemPay('capDay_rel_SSO',:)));
temp_label = cellstr(num2str(capDay_rel_SSO'));
temp_label = [temp_label; '0 (Observed)'];
dx = 20;
dy = 80;

% Values from the same tables under the baseline/observed schedule.
cf_systemPay_baseline = cfRetStruct.(['cf_systemPay_' nonltchPayModel dischargePat]);
profits_l_baseline = cf_systemPay_baseline('profits_l','output_baseline').output_baseline;
payments_l_baseline = cf_systemPay_baseline('payments_l','output_baseline').output_baseline;
tot_payments_baseline = cf_systemPay_baseline('tot_payments','output_baseline').output_baseline;

% Append values under observed to the LGRI vectors.
profits_l = [profits_l(:); profits_l_baseline];
payments_l = [payments_l(:); payments_l_baseline];
tot_payments = [tot_payments(:); tot_payments_baseline];

% LTCH profits on the y-axis (default).
fig_handle = figure; hold on;
plot(-tot_payments(1:end-1),profits_l(1:end-1),'o--b');
plot(-tot_payments(end),profits_l(end),'xr','markersize',12);
text(-tot_payments+dx, profits_l+dy, temp_label);
xlabel('Negative Medicare Payments','FontSize',18);
ylabel('LTCH Profits','FontSize',18);
title({'Counterfactual LTCH Profits vs. Medicare Payments', ...
    'Under Different LGRI Schedules', ''},'FontSize',18);
hold off;
out_file = fullfile(cfParamsStruct.outDir, 'Figures', ...
    ['cf_lgri_ltchProf_vs_medicare_post_' nonltchPayModel '_' cfParamsStruct.version '.pdf']);
print('-painters','-dpdf','-r300',out_file);
fprintf('Printed: %s\n', out_file);

% LTCH payments on the y-axis.
fig_handle = figure; hold on;
% scatter(-tot_payments,payments_l);
plot(-tot_payments(1:end-1),payments_l(1:end-1),'o--b');
plot(-tot_payments(end),payments_l(end),'xr','markersize',12);
text(-tot_payments+dx, payments_l+dy, temp_label);
xlabel('Negative Medicare Payments','FontSize',18);
ylabel('LTCH Payments','FontSize',18);
title({'Counterfactual LTCH Payments vs. Medicare Payments', ...
    'Under Different LGRI Schedules', ''},'FontSize',18);
hold off;
out_file = fullfile(cfParamsStruct.outDir, 'Figures', ...
    ['cf_lgri_ltchPay_vs_medicare_post_' nonltchPayModel '_' cfParamsStruct.version '.pdf']);
print('-painters','-dpdf','-r300',out_file);
fprintf('Printed: %s\n', out_file);

%% Output data to Excel.
if strcmpi(cfParamsStruct.version, 'v18_a3_pool')
    xlsOutFile = cfParamsStruct.xlsOutFile;
    
    temp_colnames = {'Scenario', 'Total Medicare Payments',...
        'LTCH Payments', 'LTCH Profits'};
    temp_outmat = [tot_payments(:) payments_l(:) profits_l(:)];
    temp_scenarionames = eval(['cfRetStruct.cf_systemPay_LGRI_' ...
        nonltchPayModel dischargePat '.Properties.VariableNames(:)']);
    temp_scenarionames = [strrep(temp_scenarionames,'output_',''); 'Observed Schedule'];
    xlsSheetName = ['LGRI_frontier_post',cfParamsStruct.xlsShtNm];
    xlswrite(xlsOutFile,temp_colnames,xlsSheetName, 'A1');
    xlswrite(xlsOutFile,temp_scenarionames,xlsSheetName, 'A2');
    xlswrite(xlsOutFile,temp_outmat,xlsSheetName,'B2');
    
    %print time stamp
    print_timestamp(xlsOutFile, xlsSheetName, cfParamsStruct.version, cfParamsStruct.run);

end


return