function retStruct_out = plot_policy_function(period, scenarioID, ...
    paramsStruct, retStruct, flag_outputPoliFuncPlot, cf_scenarios)


% Local parameters.
daysTopCode = 70;

% Initialize the output container.
retStruct_out = retStruct;


%%
% Prepare file/variable name suffix and plot title.
myTitle = ['Policy function for the ' period ' period'];
if isempty(scenarioID)
    fldTag=period;
    
    % The end period depends on whether we are doing counterfactuals or not
    days    = paramsStruct.days;
    TT      = paramsStruct.TT;
    backInductT = paramsStruct.backInductT;
else
    myTitle = [myTitle ': ' scenarioID];
    fldTag=[period, '_', scenarioID];
    
    % The end period depends on whether we are doing counterfactuals or not
    % Note the "_cf".
    days    = paramsStruct.days_cf;
    TT      = paramsStruct.TT_cf;
    % With counterfactual payment schedules, it's possible to have the
    % counterfactual SSO > counterfactual backInductT
    backInductT = paramsStruct.backInductT_cf;
    T       = cf_scenarios(scenarioID,'T').T;
    backInductT = max(backInductT,T);
end

% Define required variables that are local to this function.
Q_preSSO    = retStruct.(['Q_preSSO_', fldTag]);
q_postSSO   = retStruct.(['q_postSSO_', fldTag]); 
z           = retStruct.(['z_', fldTag]);   

% Health status of marginal dischargees
[Q_a, Q_s, Q_l, a_threshold, s_threshold, z_a_threshold, z_s_threshold] = ...
    policy_threshold(Q_preSSO, q_postSSO, z, backInductT, TT);


%%
if flag_outputPoliFuncPlot
    % Calculations to ensure nice axes ticks.
    myXTickMin = min(days);     myXTickMax = min(max(days),daysTopCode); % top-code days
    myXMin = myXTickMin - 0.7;  myXMax = myXTickMax + 2;
    
    fig_handle = figure;
    hold on;
    myYMin = min(z_a_threshold) - 0.7;
    myYMax = max(z_s_threshold) + 0.7;
    
    plot(days, z_a_threshold,'-b', days,z_s_threshold,'-b', 'LineWidth',2);
    xlabel('Days','FontSize',18);
    ylabel('Health Type','FontSize',18);
    title({myTitle, ''}, 'FontSize',18);
    ylim([myYMin myYMax]); xlim([myXMin myXMax]);
    text(4,max(z_s_threshold(16:end))+0.5,'Discharge to Low Acuity','FontSize',18,'LineStyle','none');
    text(4,min(z_a_threshold(16:end))-0.5,'Discharge to High Acuity','FontSize',18,'LineStyle','none');
    hold off;
    ax = gca;
    ax.XTick = myXTickMin:5:myXTickMax;

    out_file = fullfile(paramsStruct.outDir, 'Figures',  ...
        ['policy_function_' period '_' scenarioID '_' paramsStruct.version '.pdf']);
    print('-painters','-dpdf','-r300',out_file);
    fprintf('Printed %s\n', out_file);
end

%% export policy function results (only for basecase run)
if strcmpi(scenarioID, '') && strcmpi(paramsStruct.version, 'v18_a3_pool')
    temp_outmat = [days(:) z_a_threshold(:) z_s_threshold(:)];
    xlsSheetName = ['policy_functions_' period, '_',paramsStruct.xlsShtNm];
    temp_colnames = {'days' 'z_a_threshold' 'z_s_threshold'};
    
    %write xls output
    xlsOutFile=paramsStruct.xlsOutFile;
    xlswrite(xlsOutFile,temp_colnames,xlsSheetName);
    xlswrite(xlsOutFile,temp_outmat,  xlsSheetName,'A2');
    
    %print time stamp
    print_timestamp(xlsOutFile, xlsSheetName, paramsStruct.version, paramsStruct.run);
end



%%
% Populate the struct to return.
retStruct_out.(['Q_a_', fldTag])= Q_a;
retStruct_out.(['Q_s_', fldTag])= Q_s;
retStruct_out.(['Q_l_', fldTag])= Q_l;
retStruct_out.(['a_threshold_', fldTag])= a_threshold;
retStruct_out.(['s_threshold_', fldTag])= s_threshold;
retStruct_out.(['z_a_threshold_', fldTag])= z_a_threshold;
retStruct_out.(['z_s_threshold_', fldTag])= z_s_threshold;

return