function [] = plot_policy_function_overlaid(scenariosToPlot, ...
    period, cfParamsStruct, cfRetStruct)


% Local parameters.
daysTopCode = 70;
lineStyles = {'-k','-c','-g','-r','-y','-b'};
            
% Define required variables that are local to this function.
days_cf = cfParamsStruct.days_cf;
T_cf = cfParamsStruct.T_cf;

%configure plot
myYMax = 0; myYMin = 0;
myTextMax = 0; myTextMin = 0;
temp_colnames = {'days'};
temp_outmat = days_cf(:);

for scenarioID = {scenariosToPlot{:}}
    scenarioID = cell2mat(scenarioID);
    
    temp_z_a_threshold = cfRetStruct.(['z_a_threshold_' period '_' scenarioID]);
    temp_z_s_threshold = cfRetStruct.(['z_s_threshold_' period '_' scenarioID]);
    
    temp_colnames = [temp_colnames ['z_a_threshold_' scenarioID] ['z_s_threshold_' scenarioID]];
    temp_outmat = [temp_outmat temp_z_a_threshold(:) temp_z_s_threshold(:)];
    
    myYMax = max(max(myYMax, temp_z_s_threshold));
    myYMin = min(min(myYMin, temp_z_a_threshold));
    
    myTextMax = max(max(myTextMax, temp_z_s_threshold(T_cf+1:end)));
    myTextMin = min(min(myTextMin, temp_z_a_threshold(T_cf+1:end)));
    
    eval(['z_a_threshold_' scenarioID '= temp_z_a_threshold;']);
    eval(['z_s_threshold_' scenarioID '= temp_z_s_threshold;']);
end

% Overlay policy function lines.
% Each line represents one payment schedule.
myXTickMin = min(days_cf);
myXTickMax = min(max(days_cf), daysTopCode); % top-code days
myXMin = myXTickMin - 0.7;
myXMax = myXTickMax + 0.7;

myYMin = myYMin - 0.7;
myYMax = myYMax + 0.7;
myTextMin = myTextMin - 0.2;
myTextMax = myTextMax + 0.2;

iter = 1;
plot_a_str = '';
plot_s_str = '';
for scenarioID = {scenariosToPlot{:}}
    scenarioID = cell2mat(scenarioID);
    lineStyle = cell2mat(lineStyles(iter));
    plot_a_str = [plot_a_str 'days_cf, z_a_threshold_' scenarioID ', ''' lineStyle ''','];
    plot_s_str = [plot_s_str 'days_cf, z_s_threshold_' scenarioID ', ''' lineStyle ''','];
    iter = iter + 1;
end

fig_handle = figure; hold on;
plot_a_handle = eval(['plot(' plot_a_str '''LineWidth'',2)']);
plot_s_handle = eval(['plot(' plot_s_str '''LineWidth'',2)']);
hold off;

xlabel('Days','FontSize',18);
ylabel('Health Type','FontSize',18);
title({['Overlaid policy functions for the ' period '-period'], ...
        ''}, 'FontSize',18);
ylim([myYMin myYMax]); xlim([myXMin myXMax]);
text(4, myTextMax,'Discharge to Low Acuity','FontSize',18,'LineStyle','none');
text(4, myTextMin,'Discharge to High Acuity','FontSize',18,'LineStyle','none');

% Set the labels for days.
ax = gca;
ax.XTick = myXTickMin:5:myXTickMax;

% Set legend.
leg = legend(plot_a_handle, cfRetStruct.legendText, 'Location', 'east');
set(leg, 'FontSize', 18); legend boxoff;

out_file = fullfile(cfParamsStruct.outDir, 'Figures', ...
        ['policy_function_',period '_overlaid_' cfParamsStruct.version '.pdf']);
set(fig_handle, 'PaperOrientation','landscape');
set(fig_handle,'PaperUnits','normalized');
set(fig_handle,'PaperPosition', [0 0 1 1]);

print('-painters','-dpdf','-r300',out_file);
fprintf('Printed %s\n', out_file);

% Output data to Excel.
if strcmpi(cfParamsStruct.version, 'v18_a3_pool')
    xlsOutFile = cfParamsStruct.xlsOutFile;
    xlsSheetName = ['cf_polF_' period cfParamsStruct.xlsShtNm];
    xlswrite(xlsOutFile,temp_colnames,xlsSheetName);
    xlswrite(xlsOutFile,temp_outmat,xlsSheetName,'A2');
    
    %print time stamp
    print_timestamp(xlsOutFile, xlsSheetName, cfParamsStruct.version, cfParamsStruct.run);

end

clear temp_* endMsg;

return