function [W_postSSO_prime, q_postSSO, V_A, V_S, l] = post_value_function(...
    params_pref, param_scale, repCostPerDay, ...
    z, P_D_struct, p_postSSO, paramsStruct)

% Local params to control value function
valFuncCeil  = paramsStruct.valFuncCeil;
valFuncFloor = paramsStruct.valFuncFloor_post;

% Extract from paramsStruct
n   = paramsStruct.n;
backInductT   = paramsStruct.backInductT; % The day when health becomes a stationary process


%params preferences
beta = paramsStruct.beta;
gamma = params_pref(1);
v1A = params_pref(2);
v0S = params_pref(3);
v1S = params_pref(4);

%NPV of non budgetary factors (to be estimated)
V_A = zeros(1,n);
V_A(1,2:end) = v1A*z;

V_S = zeros(1,n);
V_S(1,2:end) = v0S + v1S*z;

% LTCH's reported cost (data)
c = ones(1,n)*repCostPerDay;

% vary cost for robustness exercise
if isfield(paramsStruct, 'varyCost')
    if paramsStruct.varyCost==1
        c = [0,paramsStruct.varyCost_A+paramsStruct.varyCost_B*z];
        gamma = 1;
    end
end

%Initialize
W_post = [0 linspace(6e4,9e4,n-1)];
W_postSSO_prime = zeros(1,n);
q_postSSO = zeros(n,3);


%Control estimation
tol = 1e-6;
err = 10;

% Even in the model with time-varying health, we force the model to have
% stationary health process for t>=backInductT. So we just extract
% P_D_(backInductT) from P_D_struct (an input argument).
P_D = P_D_struct.(['P_D_', num2str(backInductT)]);

%LTCH flow
flow = p_postSSO - gamma*c(1,:);

% Value function iteration
trim = 1;
while err > tol    
   
    %log sum formula
    [W_prime, Q_prime, ~, a, s, l]  = ...
    log_sum(V_A, V_S, flow, W_post(1,:), P_D, beta, param_scale, trim, valFuncCeil, valFuncFloor);

    W_postSSO_prime(1,2:end) = W_prime;
    q_postSSO(2:end,:)       = Q_prime;

    % error and update
    err = (W_postSSO_prime - W_post)*(W_postSSO_prime - W_post)';    
    W_post = W_postSSO_prime;
end


%output well behave V_A/V_S for identification exercise
if isfield(paramsStruct, 'plotV')
    if paramsStruct.plotV==1
        V_A=a;
        V_S=s;
    end
end