function [W_preSSO, Q_preSSO, l_preSSO]  = pre_value_function(...
    params_pref, param_scale, repCostPerDay, ...
    W_postSSO_prime, z, P_D_struct, p_preSSO, paramsStruct)

% Local params to control value function
valFuncCeil  = paramsStruct.valFuncCeil;
valFuncFloor = paramsStruct.valFuncFloor_pre;

% Extract from paramsStruct
n   = paramsStruct.n;
backInductT   = paramsStruct.backInductT; % The day when health becomes a stationary process
T  = paramsStruct.T; % SSO day

%params preferences
beta = paramsStruct.beta;
gamma = params_pref(1);
v1A = params_pref(2); 
v0S = params_pref(3); 
v1S = params_pref(4); 

%NPV of non budgetary factors (to be estimated)
V_A = zeros(1,n);
V_A(1,2:end) = v1A*z;

V_S = zeros(1,n);
V_S(1,2:end) = v0S + v1S*z;

% LTCH's reported cost (data)
c = ones(1,n)*repCostPerDay; 

% vary cost for robustness exercise
if isfield(paramsStruct, 'varyCost')
    if paramsStruct.varyCost==1
        c = [0,paramsStruct.varyCost_A+paramsStruct.varyCost_B*z];
        gamma = 1;
    end
end

%Initialize
W_preSSO = zeros(backInductT-1,n);      
Q_preSSO = zeros(backInductT-1,n,3);    
l_preSSO = zeros(backInductT-1,n);      

%%%----- Period backInductT - 1: backInductT is the day when everything
% become stationary. On the other hand, T is the SSO threshold day, i.e.
% Day 0. We used to always do backward induction from the SSO day, but
% since we are now allowing some health processes to be time-varying past
% the SSO, we need a later T to start doing this backward induction.

% We need these conditions because model types a5, a7 have a time-varying
% health process up to t = backInductT >= T, after which we force it to become
% stationary.
P_D = extract_PD(P_D_struct,paramsStruct.estType,backInductT,backInductT-1);

% Pad the preSSO vector if there is a mismatch between backInductT and T.
% For example, in model a7, we allow the health process to vary with t
% beyond T (i.e. backInductT > T), so the new "preSSO" should include the
% flows of 0's up to and include day backInductT.
% length(p_preSSO) will indicate the non-zero flows before a cap day; this
% flexibility allows the counterfactual payment schedules with different
% cap days (new SSO) to work.
if backInductT > length(p_preSSO)
    p_preSSO = [p_preSSO; zeros(backInductT-T,1)];
end

%LTCH flow revenue
flow = p_preSSO(backInductT) - gamma*c(1,:);

%nuemrical adjustment
trim = paramsStruct.valFuncTrim_back0;

%log sum formula
[W_prime, Q_prime, l_prime, ~, ~, ~]  = ...
    log_sum(V_A, V_S, flow, W_postSSO_prime(1,:), P_D, beta, param_scale, trim, valFuncCeil, valFuncFloor);
W_preSSO(backInductT-1,2:end)   = W_prime;
Q_preSSO(backInductT-1,2:end,:) = Q_prime;
l_preSSO(backInductT-1,2:end)   = l_prime;

%%%----- Periods before backInductT-1
% backInductT (>= T) is the day the health process becomes stationary.
trim = 1;
for t = backInductT-2:-1:1
    % We need these conditions because model type a5 has a time-varying
    % health process up to t = backInductT >= T, after which we force it to
    % become stationary.
    P_D = extract_PD(P_D_struct,paramsStruct.estType,backInductT,t);
    
    %LTCH flow revenue
    flow = p_preSSO(t+1) - gamma*c(1,:);
    
    % log sum formula
    [W_prime, Q_prime, l_prime, ~, ~, ~]  = ...
    log_sum(V_A, V_S, flow, W_preSSO(t+1,:), P_D, beta, param_scale, trim, valFuncCeil, valFuncFloor);
            
    % record results
    W_preSSO(t,2:end)   = W_prime;
    Q_preSSO(t,2:end,:) = Q_prime;  
    l_preSSO(t,2:end)   = l_prime; 
end
