%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run one counterfactual (or baseline) scenario
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function cfRetStruct_out = run_counterfactual(scenarioID, cfParamsStruct, cfRetStruct, T_cf)


    % Initialize the output container.
    cfRetStruct_out = cfRetStruct;

    %%
    % Define required variables that are local to this function.
    n_cf = cfParamsStruct.n_cf;
    TT_cf = cfParamsStruct.TT_cf;
    % With counterfactual payment schedules, it's possible to have the
    % counterfactual SSO > counterfactual backInductT
    backInductT_cf = max(cfParamsStruct.backInductT_cf,T_cf);

    % We need this hack because the functions below will extract these names
    % from cfParamsStruct.
    cfParamsStruct.T  = T_cf;
    cfParamsStruct.backInductT = backInductT_cf;
    cfParamsStruct.n  = n_cf;
    cfParamsStruct.TT = TT_cf;

    % Select the preSSO and postSSO payment schedules based on the scenario and period.
    cfParamsStruct.p_preSSO_post  = cfRetStruct.(['p_preSSO_'  scenarioID]);
    cfParamsStruct.p_postSSO_post = cfRetStruct.(['p_postSSO_' scenarioID]);

    %% dynamic or static LOS model
    if ~strcmp(cfParamsStruct.estType, 'a9')
        [share_a_post_hat_cf, share_s_post_hat_cf, share_l_post_hat_cf, share_m_post_hat_cf, ...
            z_post, P_D_post, ~, ...
            ~, q_postSSO_post, ~, ~, ~, ...
            ~, Q_preSSO_post, ~, ...
            h_a_post, h_s_post, h_a_dist_post, h_s_dist_post, ...
            H_post, H_l_post] = ...
                evaluate_objective('post', cfRetStruct.params_all_hat, cfParamsStruct);   
    else
        %post-PPS moments
        [share_a_post_hat_cf, share_s_post_hat_cf, share_m_post_hat_cf, share_l_post_hat_cf, ...
         z_post, P_D_post, ~, h_a_post, h_s_post, h_a_dist_post, h_s_dist_post, ...
         H_post, H_l_post]= ...
            static_simulation_forward('post', cfRetStruct.params_all_hat, cfParamsStruct);
        q_postSSO_post=[];
        Q_preSSO_post =[];
    end
    %% Populate cfRetStruct_out.
    % Mainly for counterfactual plots and systemwide payments.
    cfRetStruct_out.(['share_a_post_' scenarioID])= share_a_post_hat_cf;
    cfRetStruct_out.(['share_s_post_' scenarioID])= share_s_post_hat_cf;
    cfRetStruct_out.(['share_l_post_' scenarioID])= share_l_post_hat_cf;
    cfRetStruct_out.(['share_m_post_' scenarioID])= share_m_post_hat_cf;

    cfRetStruct_out.(['h_a_post_' scenarioID])= h_a_post;
    cfRetStruct_out.(['h_s_post_' scenarioID])= h_s_post;
    cfRetStruct_out.(['h_a_dist_post_' scenarioID])= h_a_dist_post;
    cfRetStruct_out.(['h_s_dist_post_' scenarioID])= h_s_dist_post;

    % Mainly for policy function plots.
    cfRetStruct_out.(['z_post_' scenarioID])  = z_post;
    cfRetStruct_out.(['P_D_post_' scenarioID])= P_D_post;
    cfRetStruct_out.(['q_postSSO_post_' scenarioID]) = q_postSSO_post;
    cfRetStruct_out.(['Q_preSSO_post_' scenarioID])  = Q_preSSO_post;
    cfRetStruct_out.(['H_post_' scenarioID])  = H_post;
    cfRetStruct_out.(['H_l_post_' scenarioID])= H_l_post;

    % Get counterfactual discharge shares for this combination of period and
    % scenario, then calculate discharge PDF and hazard.
    share_l_cf   = cfRetStruct_out.(['share_l_post_' scenarioID]);
    d_share_l_cf = -diff([1; share_l_cf]);                  % discharge PDF

    % Populate cfRetStruct_out with the discharge PDF and hazard, and % of
    % patients never discharged from the LTCH.
    cfRetStruct_out.(['d_share_l_post_' scenarioID])= d_share_l_cf;
    pct_l_never_disc_cf = (1-sum(d_share_l_cf))*100;
    cfRetStruct_out.(['pct_l_never_disc_post_' scenarioID])= pct_l_never_disc_cf;
    fprintf('Scenario=%s, never discharged=%0.2f%%\n', scenarioID,pct_l_never_disc_cf);

end