function [share_a, share_s, share_l, share_m, ...
        h_a, h_s, h_a_dist, h_s_dist, ...
        H, H_l, Q_a, Q_s, Q_l] = ...
    simulate_forward(q_postSSO,Q_preSSO,z,P_D_struct,h_0,paramsStruct)

%share's simulated to different locations with probabilistic discharge functions


%%
% global params_grid

n   = paramsStruct.n;
TT  = paramsStruct.TT;
backInductT   = paramsStruct.backInductT; % The day when health becomes a stationary process

%initial health
H_l = zeros(TT + 1, n); 
H = zeros(TT + 1, n);   

H_l(1,2:end) = h_0';    % Initial health on the day BEFORE the start of the model framework.
H(1,2:end) = h_0';

%initialize
share_a = zeros(TT,1);
share_s = zeros(TT,1);
share_l = zeros(TT,1);
share_m = zeros(TT,1);

h_a = zeros(TT,1);
h_s = zeros(TT,1);
h_a_dist = zeros(TT,n-1);   % n-1 is the dimension of z. We reserve one 
                            % grid point as the absorbing state of death.
h_s_dist = zeros(TT,n-1);

%Discharge functions
Q_a = [squeeze(Q_preSSO(:,:,1));  ones(TT-backInductT+1,1)*q_postSSO(:,1)'];
Q_s = [squeeze(Q_preSSO(:,:,2));  ones(TT-backInductT+1,1)*q_postSSO(:,2)'];
Q_l = [squeeze(Q_preSSO(:,:,3));  ones(TT-backInductT+1,1)*q_postSSO(:,3)'];

%Simulate forward

%Switch ordering: 
%1. People die
%2. Remaing people get discharged to A,S, or retained
%3. Retained go to top of loop

for t = 1:TT
    % We need these conditions because model type a5 has a time-varying
    % health process up to t = backInductT >= T, after which we force it to
    % become stationary.   
    P_D = extract_PD(P_D_struct,paramsStruct.estType,backInductT,t);
    
    %share of people who die at 3 am
    share_m(t) = H_l(t,:)*P_D(:,1);
    
    %health dist of remaining patients at 3 am
    H_evol = H_l(t,:)*P_D;
        
    %shares discharged to ACH, SNF, retained at noon
    share_a(t) = Q_a(t,:)*H_evol';
    share_s(t) = Q_s(t,:)*H_evol';
    share_l(t) = Q_l(t,:)*H_evol';
    
    %average health type of discharged individuals by day
    h_a(t) = sum((Q_a(t,2:end).*H_evol(1,2:end)).*z)/sum(Q_a(t,2:end).*H_evol(1,2:end)); 
    h_s(t) = sum((Q_s(t,2:end).*H_evol(1,2:end)).*z)/sum(Q_s(t,2:end).*H_evol(1,2:end));

    %distribution of health type of discharged individuals by day
    h_a_dist(t,:) = (Q_a(t,2:end).*H_evol(1,2:end))/sum(Q_a(t,2:end).*H_evol(1,2:end));
    h_s_dist(t,:) = (Q_s(t,2:end).*H_evol(1,2:end))/sum(Q_s(t,2:end).*H_evol(1,2:end));
     
    %conditional distribution for next day at 1 am
    H_l(t+1,:) = Q_l(t,:).*H_evol(1,:);
    
    %unconditional distribution
    H(t+1,:) = H(t,:)*P_D;   
end
%%
return
