%static LOS models, as comparisons to LTCH

function [share_a, share_s, share_m, share_l, ...
            z, P_D, h_0, ...
            h_a, h_s, h_a_dist, h_s_dist, ...
            H, H_l] ...
            = static_simulation_forward(period, params_all, paramsStruct)


%% parameter set up
[params_health_pre, params_health_post, params_pref, param_scale, ~, ~, ~, ~] ...
          = assign_params(params_all, paramsStruct);
      
%estiamted parameters
if strcmp(period, 'pre')
    params_health  = params_health_pre;
    p_preSSO       = paramsStruct.p_preSSO_pre;
    p_postSSO      = paramsStruct.p_postSSO_pre;
    repCostPerDay  = paramsStruct.repCostPerDay_pre;
elseif strcmp(period, 'post')
    params_health = params_health_post;        
    p_preSSO      = paramsStruct.p_preSSO_post;
    p_postSSO     = paramsStruct.p_postSSO_post;
    repCostPerDay = paramsStruct.repCostPerDay_post;        
end
        
        
gamma=params_pref(1);
v1A=params_pref(2);
v0S=params_pref(3);
v1S=params_pref(4);

%grid size and time
n   = paramsStruct.n;
TT  = paramsStruct.TT;
backInductT=paramsStruct.backInductT;
T   = paramsStruct.T;
%% Health process

[z, P_D, h_0] = health_process(params_health, paramsStruct);
P_D = extract_PD(P_D,paramsStruct.estType,backInductT);

%% Discrete Choice of discharge

%value for upstream/downstream
a = repmat([0, v1A*z], TT,1);
s = repmat([0, v0S + v1S*z], TT,1);

% LTCH's flow ( pay - reported cost (data))
if backInductT > length(p_preSSO)
    p_preSSO = [p_preSSO; zeros(backInductT-T,1)];
end
l = [p_preSSO; p_postSSO.*ones(TT-backInductT,1)] - gamma*repCostPerDay*ones(TT,1);


%% LOS model, discounted payoff of each LOS X destination pair

%local hardcoded parameters
beta = paramsStruct.beta;

%for each initial health status; (n-1): the 1st one is death
los=zeros(n-1,TT*2);
for i=1:n-1      
    hi=zeros(n,1); 
    hi(i+1)=1;   %point mass at i

    %health distribution over time
    H = zeros(TT, n);   
    H(1,:)=hi'*P_D;     %health at day 1
    for t=2:TT
        H(t,:) = H(t-1,:)*P_D;   
    end

    %prob adjusted discounted up/down payoff & flow: beta*E(a|t)
    a_prob=sum(H.*a,2).*(beta.^(1:TT)');
    s_prob=sum(H.*s,2).*(beta.^(1:TT)');

    %prob adjusted l: discounted and considering the prob of death, then sum cumulative DCF
    l_cum=cumsum(  l.*(beta.^(1:TT)').*sum(H(:,2:end),2)  );    

    %discounted payoff
    dis_los=(   [a_prob+l_cum;...   %discharge upstream each day + cum payoff in LTCH til each day
                s_prob+l_cum]...    %discharge downstream each day + cum payoff in LTCH til each day
                )./param_scale;

    %logit probability
    los(i,:)=exp(dis_los)./sum(exp(dis_los));
end

LOS_td=reshape(los, size(los,1), size(los,2)/2,2);

%combine prob of LOS up and LOS down
LOS_t=sum(LOS_td,3);  
    

%% simulate forward

%initial health
H_l = zeros(TT + 1, n); 
H_l(1,2:end) = h_0';    % Initial health on the day BEFORE the start of the model framework.

H = zeros(TT + 1, n);  
H(1,2:end) = h_0';

%initialize
share_a = zeros(TT,1);
share_s = zeros(TT,1);
share_l = zeros(TT,1);
share_m = zeros(TT,1);

%health post discharge
h_a = zeros(TT,1);
h_s = zeros(TT,1);
h_a_dist = zeros(TT,n-1);
h_s_dist = zeros(TT,n-1);

%Simulate forward
for t = 1:TT
    %prob transition till t from initial stage
    PDt = P_D^t;
    PDt = PDt(2:end,2:end);

    %discharge shares = supposed discharge X survivial rate till t
    surviv = sum(PDt,2);
    LOS_stay = sum(LOS_t(:,t+1:TT),2);

    share_a(t) = h_0'*(LOS_td(:,t,1).*surviv);
    share_s(t) = h_0'*(LOS_td(:,t,2).*surviv);
    share_l(t) = h_0'*(LOS_stay.*surviv);

    %ppl who die at t
    share_m(t) = H_l(t,:)*P_D(:,1);

    %distribution of health type of discharge individuals by day
    h_a_dist(t,:) = (h_0.*LOS_td(:,t,1))'*PDt/share_a(t);
    h_s_dist(t,:) = (h_0.*LOS_td(:,t,2))'*PDt/share_s(t);

    %average health type of discharged individuals by day
    h_a(t)  = sum((z'.*h_0.*LOS_td(:,t,1))'*PDt)/share_a(t);
    h_s(t)  = sum((z'.*h_0.*LOS_td(:,t,2))'*PDt)/share_s(t);


    %conditional distribution for those retaim at LTCH 
    H_l(t+1,:) = [0, (h_0.*LOS_stay)'*PDt];        

    %unconditional distribution
    H(t+1,:)   = H(t,:)*P_D;
end
