%take parameters and simulate all moments
%export moments (both vecterized and in matrix)
% as well as post-PPS health distribution, transition matrix, etc..
function retStruct = ...
        store_estimation_results(paramsStruct, retStruct, params_all_hat, sse_hat, exitflag)


    retStruct.sse_hat  = sse_hat;
    retStruct.exitflag = exitflag;

    % parameters
    [params_health_pre, params_health_post, params_pref, param_scale, ...
              zeta_a_pre, zeta_s_pre, zeta_a_post, zeta_s_post] ...
              = assign_params(params_all_hat, paramsStruct);

    % objective outputs
    [moments_hat, ...
        share_a_pre_hat, share_s_pre_hat, share_l_pre_hat, share_m_pre_hat, ...
        z_pre, P_D_pre, h_0_pre, ...
        W_postSSO_prime_pre, q_postSSO_pre, V_A_pre, V_S_pre, ...
        Q_preSSO_pre, ...
        h_a_pre, h_s_pre, h_a_dist_pre, h_s_dist_pre, ...
        H_pre, H_l_pre, Q_a_pre, Q_s_pre, Q_l_pre, ...
        share_a_post_hat, share_s_post_hat, share_l_post_hat, share_m_post_hat, ...
        z_post, P_D_post, h_0_post, ...
        W_postSSO_prime_post, q_postSSO_post, V_A_post, V_S_post, V_l_post, ...
        W_preSSO_post, Q_preSSO_post, l_preSSO_post, ...
        h_a_post, h_s_post, h_a_dist_post, h_s_dist_post, ...
        H_post, H_l_post, Q_a_post, Q_s_post, Q_l_post, ...
        nonltchPay_a_pre_hat,  nonltchPay_s_pre_hat, ...
        nonltchPay_a_post_hat, nonltchPay_s_post_hat ...            
        ] = collect_obj_results(params_all_hat, paramsStruct);
    
    % store parameters
    retStruct.params_all_hat     = params_all_hat;
    retStruct.params_health_pre  = params_health_pre;
    retStruct.params_health_post = params_health_post;
    retStruct.params_pref = params_pref;
    retStruct.param_scale = param_scale;

    if strcmpi(paramsStruct.objType, 'discShares_nonltchPay') 
        if ~paramsStruct.ind_omitPrePPSNonLtch
            retStruct.params_zeta_a_pre = zeta_a_pre;
            retStruct.params_zeta_s_pre = zeta_s_pre;
            retStruct.nonltchPay_a_pre_hat = nonltchPay_a_pre_hat;
            retStruct.nonltchPay_s_pre_hat = nonltchPay_s_pre_hat;
        end
        retStruct.params_zeta_a_post = zeta_a_post;
        retStruct.params_zeta_s_post = zeta_s_post;
        retStruct.nonltchPay_a_post_hat = nonltchPay_a_post_hat;
        retStruct.nonltchPay_s_post_hat = nonltchPay_s_post_hat;
    end


    % store standard error
    if strcmp(paramsStruct.momentsUsed, 'pre_post') && strcmp(paramsStruct.objType, 'discShares_nonltchPay')
        retStruct.jacMat = paramsStruct.jacMat;
        retStruct.gmmSE  = paramsStruct.gmmSE;
        retStruct.gmmVar = paramsStruct.gmmVar;
    end    

    % intermediate results from objective function
    retStruct.moments_hat = moments_hat;

    retStruct.z_pre = z_pre;
    retStruct.P_D_pre = P_D_pre;
    retStruct.h_0_pre = h_0_pre;
    retStruct.W_postSSO_prime_pre = W_postSSO_prime_pre;
    retStruct.q_postSSO_pre = q_postSSO_pre;
    retStruct.V_A_pre = V_A_pre;
    retStruct.V_S_pre = V_S_pre;
    retStruct.Q_preSSO_pre = Q_preSSO_pre;
    retStruct.share_a_pre_hat = share_a_pre_hat;
    retStruct.share_s_pre_hat = share_s_pre_hat;
    retStruct.share_l_pre_hat = share_l_pre_hat;
    retStruct.share_m_pre_hat = share_m_pre_hat;
    retStruct.h_a_pre = h_a_pre;
    retStruct.h_s_pre = h_s_pre;
    retStruct.h_a_dist_pre = h_a_dist_pre;
    retStruct.h_s_dist_pre = h_s_dist_pre;
    retStruct.H_pre = H_pre;
    retStruct.H_l_pre = H_l_pre;
    retStruct.Q_a_pre = Q_a_pre;
    retStruct.Q_s_pre = Q_s_pre;
    retStruct.Q_l_pre = Q_l_pre;

    retStruct.z_post = z_post;
    retStruct.P_D_post = P_D_post;
    retStruct.h_0_post = h_0_post;
    retStruct.W_postSSO_prime_post = W_postSSO_prime_post;
    retStruct.q_postSSO_post = q_postSSO_post;
    retStruct.V_A_post = V_A_post;
    retStruct.V_S_post = V_S_post;
    retStruct.W_preSSO_post = W_preSSO_post;
    retStruct.Q_preSSO_post = Q_preSSO_post;
    retStruct.share_a_post_hat = share_a_post_hat;
    retStruct.share_s_post_hat = share_s_post_hat;
    retStruct.share_l_post_hat = share_l_post_hat;
    retStruct.share_m_post_hat = share_m_post_hat;
    retStruct.h_a_post = h_a_post;
    retStruct.h_s_post = h_s_post;
    retStruct.h_a_dist_post = h_a_dist_post;
    retStruct.h_s_dist_post = h_s_dist_post;
    retStruct.H_post = H_post;
    retStruct.H_l_post = H_l_post;
    retStruct.Q_a_post = Q_a_post;
    retStruct.Q_s_post = Q_s_post;
    retStruct.Q_l_post = Q_l_post;
    
end